      SUBROUTINE EMBED (ISSUE)
C***********************************************************************
C   THIS VERSION OF EMBED (JULY 1980) WAS PIECED TOGETHER OUT OF ROU-   
C   TINES WRITTEN BY GORDON CRIPPEN, IRWIN KUNTZ, TODD NORDLAND AND     
C   TIM HAVEL, AND WAS IMPLEMENTED BY THE LAST AUTHOR ON THE LAWRENCE-  
C   BERKELEY LABORATORY'S CDC 7600 USING THEIR VERSION OF THE MINNESOTA 
C   FORTRAN COMPILER. SINCE THIS FORTRAN IS FAIRLY STANDARD, NO GREAT   
C   PROBLEMS SHOULD ARISE IN ATTEMPTING TO IMPLEMENT IT ON OTHER MACH-  
C   INCES AND COMPILERS. POSSIBLE REQUIRED CHANGES ARE AS FOLLOWS: THE  
C   INTRINSIC RANDUM NUMBER GENERATOR 'RANF' USED IN DSTMAT AND EIGENV  
C   MUST BE CHANGED TO WHATEVER IS AVAILABLE LOCALLY; THE 'ENCODE' USED 
C   FOR VARIABLE FORMATING IN EIGENV IS NONSTANDARD AND SUPERFLUOUS IN
C   THIS CONTEXT, AND MAY BE DISCARDED IF NECESSARY; SINCE THE CDC 7600
C   PACKS 10 CHARACTERS PER WORD, WHICH IS MORE THAN MOST MACHINES, THE 
C   HANDELING OF CHARACTER STRINGS MAY HAVE TO BE CHANGED, ESPECIALLY   
C   FOR THE VARIABLE 'STATUS' IN SCCG. ALL OF THE REQUIRED PARAMETERS,  
C   WORKSPACE, INPUT DATA AND OUTPUT ARE PASSED TO & FROM EMBED IN COM- 
C   MON, ITS ONLY ARGUMENT BEING THE SYMBOLIC PROBLEM NAME. ONCE INSIDE 
C   EMBED, HOWEVER, THINGS ARE PASSED TO LOWER-LEVEL ROUTINES AS ARGU-  
C   MENTS VARIABLY DIMENSIONED AS 'NDIM'. THE ONLY EXCEPTION TO THIS    
C   RULE OCCURS IN THE OPTIMIZATION ROUTINE 'SCCG' AND THE FUNCTIONS IT 
C   ACTS UPON. HERE ONLY THE VARIABLES ARE PASSED AS ARGUMENTS, THE     
C   PARAMETERS AND WORKSPACE BEING IN COMMON. GOOD LUCK!                
C***********************************************************************
C
C***********************************************************************
C	MODIFIED 9/80-10/80 BY D.H. SMITH FOR DEC-10 COMPATIBILITY
C***********************************************************************
C
C
C   INPUT:   A SET OF UPPER & LOWER BOUNDS ON THE INTERPOINT DISTANCES,
C            TOGETHER WITH A SET OF ASSIGNED CHIRALITIES TO ANY CHIRAL
C            CENTERS. NECESSARY PARAMETERS ARE DEFINED BELOW.
C   OUTPUT:  A SET OF 3D COORDINATES WHOSE DISTANCES LIE WITHIN THE
C            PRESCRIBED BOUNDS & WHOSE CHIRAL CENTERS HAVE THE ASSIGNED
C            CHIRALITIES IF SUCH A SET EXISTS, OR A SET OF 'BEST FIT'
C            COORDINATES IF NOT.
C   RETURNS: 1 IF THE 'EMBEDDING' WAS SUCCESSFUL, -1 IF EMBEDDING WAS
C            NOT POSSIBLE, & ZERO IF A DECISION COULD NOT BE REACHED
C            WITHIN THE LIMIT ON THE NO. OF OPTIMIZATION ITERATIONS.
C
      EXTERNAL ERRFCT,CHIRER
      LOGICAL OK
      INTEGER EMBED1,EMBED2
      DIMENSION EVL(3),TVL(3)
      COMMON /INFO/ TITLE(16),LTITLE,IOPN1
C
      COMMON /XYZS/ NATOM,XYZ(3,72)
C   THE NO. OF ATOMS OR RESIDUES, AND THE OUTPUT COORDINATES.
C
      COMMON /BNDS/ NPNTS,BND(72,72)
C   THE NO. OF GEOMETRIC POINTS = NATOM + NO. OF OUTRIGGERS, AND THE
C   UPPER (ABOVE DIAGONAL) & LOWER (BELOW DIAGONAL) DISTANCE BOUNDS.
C   NB: THESE BOUNDS ARE RETURNED SQUARED AFTER CALLING EMBED.
C
      COMMON /CHIR/ NCHIR,LIG(4,120),VAL(120)
C   THE NO. OF CHIRAL CENTERS, THE ATOMS THEREOF, AND WORKSPACE.
C
      COMMON /EMBD/ DUMP,NDIM,NTRY,SMALL,DWK(72),EWK(72,72)
      LOGICAL DUMP
C   IF DUMP = .FALSE., NO OUTPUT EXCEPT ERROR MESSAGES IS PRODUCED.
C   NDIM = ARRAY DIMENSIONS = MAX. NO. OF POINTS TO BE USED.
C   NTRY = NO. OF ATTEMPTS TO PRODUCE VIABLE TRIAL COORDINATES.
C   SMALL = MIN. RATIO FCT. VALUE TO GRAD. NORM FOR NONEMBEDDABILITY.
C   DWK AND EWK ARE REQUIRED WORKSPACE.
C
      COMMON /SCCGC/ WK1(3,72),WK2(3,72),WK3(3,72),GRD(3,72)
C   WORKSPACE FOR EMBED AND SCCG, AND GRADIENT OF ERROR FUNCTIONS.
C
      COMMON /PARA/ GESFCT,STPMIN,STPMAX,CAPPA,DELTA,EPSLN,
     &              FAST,SLOW,ANGMAX,INTMAX,RESTAR
      INTEGER RESTAR
      COMMON /CTRL/ SLFCOR,IFREQ,ITNLIM,FCTMIN,GRDMIN
      LOGICAL SLFCOR
C   PARAMETERS FOR THE OPTIMIZER SCCG; DEFINED IN THAT ROUTINE.
C
      DIMENSION LABEL1(10),LABEL2(5)
C
C	DIMENSION IERROR FOR WORD LENGTH COMPATABILITY
C
      DIMENSION IERROR(2)
      DATA (LABEL1(I),I=1,10) /4HTRIA,4HNGLE,4H SMO,4HOTHE,4HD BO,
     &                         4HUNDS,4H TO ,4HEMBE,4HD   ,4H    /
      DATA (LABEL2(I),I=1,5)  /4H AFT,4HER O,4HPTIM,4HIZAT,4HION /
      ISTRCT = 0
      IF (DUMP) WRITE(20,10) (TITLE(NNN),NNN=1,16)
C
C   INTITIALIZE THE CHIRALITY CONSTRAINTS; ENTRY EMBED1 SKIPS THIS.
C
      IF (NCHIR.GT.0) CALL CHIRIN (DUMP,NDIM,NCHIR,LIG,BND,VAL)
      ENTRY EMBED1 (ISSUE)
      ISTRCT = ISTRCT + 1
C
C   USE THE TRIANGLE AND INVERSE TRIANGLE INEQUALITIES TO SMOOTH BOUNDS.
C
      CALL TRNGL (DUMP,NDIM,NPNTS,10,BND)
      CALL TRINV (DUMP,NDIM,NPNTS,10,BND)
      IF (DUMP) CALL GRAFIC (NDIM,NPNTS,BND,LABEL1)
      GO TO 2
C
C   IF WE USE ENTRY EMBED2, THE BOUNDS ARE ASSUMED SQUARED BUT OTHERWISE
C   READY TO USE. THUS EMBED2 MAY BE USED TO GENERATE ADDITIONAL STRUCT-
C   URES WITHOUT WASTING TIME ON MULTIPLE PASSES THROUGH TRNGL & TRINV.
C
      ENTRY EMBED2 (ISSUE)
      ISTRCT = ISTRCT + 1
      NM1 = NPNTS-1
      DO 1 I=1,NM1
      IP1 = I+1
      DO 1 J=IP1,NPNTS
      BND(I,J) = SQRT(BND(I,J))
      BND(J,I) = SQRT(BND(J,I))
    1 CONTINUE
C
C   SET VARAIBLES FOR USE IN TRIAL COORDINATE GENERATION; IF WE ARE
C   USING THE SELF-CORRECTION OPTION & OUTRIGGERS, WK2 MUST BE SET
C   TO THE COORDINATES OF THE OUTRIGGERS FOR USE IN SCCG/COEFCT.
C
    2 RMS = 1.E35
C
C	RMS WAS 1.E98 TO REPRESENT A LARGE NUMBER - WE CAN ONLY
C	GO TO E38 ON THE PDP-10,SO I MADE IT SLIGHTLY SMALLER...
C
      DO 3 I=1,3
      DO 3 J=1,NPNTS
      WK1(I,J) = XYZ(I,J)
      IF (SLFCOR .AND. J.GT.NATOM) WK2(I,J) = XYZ(I,J)
    3 CONTINUE
C
C   GENERATE TRIAL COORDINATES FOR OPTIMIZATION USING THE PROCEDURE OF
C   CRIPPEN AND HAVEL, ACTA CRYST (1978) A34 282-4. WE REPEAT THE REL-
C   ATIVELY CHEAP METRIC MATRIX GENERATION UNTIL WE GET < 2 NEGATIVE
C   SQUARED DISTANCES TO THE CENTER OF MASS, AND WE TRY UP TO NTRY
C   TIMES TO GET THE EIGENVALUES TO BE POSITIVE AND IN ORDER.
C
      DO 5 ITRY=1,NTRY
    4 CALL DSTMAT (NDIM,NPNTS,EWK,BND)
      IF (DUMP) WRITE (20,80)
   80 FORMAT (/,' STARTING DISTANCE MATRIX')
      IF (DUMP) CALL MATPRT (EWK,NDIM,NPNTS)
      CALL METRIC (DUMP,NDIM,NPNTS,DWK,EWK,NNEG)
      IF (NNEG.GT.1) GO TO 4
      IF (DUMP) WRITE(20,90)
   90 FORMAT (/,' METRIC MATRIX')
      IF (DUMP) CALL MATPRT (EWK,NDIM,NPNTS)
      CALL EIGENV (DUMP,NDIM,NPNTS,3,100,1.E-6,TVL,WK1,DWK,EWK,OK)
      CALL COORDS (DUMP,OK,NDIM,NATOM,NPNTS,(NTRY-ITRY),
     &             BND,TVL,EVL,WK1,XYZ,RMS)
      IF (OK) GO TO 6
    5 CONTINUE
      WRITE (20,20) (EVL(I),I=1,3),RMS
C
C   OPTIMIZE, 1ST ON THE CHIRALITY CONSTRAINTS ALONE AS GETTING THESE
C   RIGHT TO BEGIN WITH GREATLY SPEEDS THINGS, AND THEN ON THE TOTAL
C   ERROR FUNCTION WHICH GOES TO ZERO ONLY WHEN BOTH CHIRALITY AND
C   DISTANCE CONSTRAINTS HAVE BEEN MET.
C
    6 IF (DUMP) CALL XDSHOW (EWK,XYZ,NDIM,NPNTS)
      IF (NCHIR.LE.0) GO TO 7
      CALL SCCG (CHIRER,NATOM,XYZ,NFCT,NITN,FCTVAL,GRDNRM,IERROR,DUMP)
      IF (DUMP) CALL XDSHOW (EWK,XYZ,NDIM,NPNTS)
    7 CALL SCCG (ERRFCT,NATOM,XYZ,NFCT,NITN,FCTVAL,GRDNRM,IERROR,DUMP)
C
      IF (.NOT.DUMP) GO TO 8
      WRITE (20,30) ISTRCT
      WRITE (20,40) NITN,NFCT,FCTVAL,GRDNRM
      IF ((IERROR(1).NE.'     ' .AND. IERROR(2).NE.'     ') .AND.
     1(IERROR(1).NE.' OVER' .AND. IERROR(2).NE.'LIMIT')) 
     2          WRITE (20,50) (IERROR(NNN), NNN=1,2)
      WRITE (20,60)
      WRITE (20,70) (J,(XYZ(I,J),I=1,3),J=1,NPNTS)
      CALL RMSERS (DUMP,.TRUE.,NDIM,NPNTS,XYZ,BND,LABEL2,
     &                       RMS,HI,IHI,JHI,HO,IHO,JHO)
      CALL DMDUMP (NDIM,NPNTS,XYZ,BND,EWK)
C
C   IF  FCTVAL < FCTMIN, THE BOUNDS ARE ASSUMED TO BE EMBEDDABLE;
C   ELSE IF THE GRADIENT IS NOT 'SMALL' COMPARED TO THE ERROR, THE
C   SITUATION IS INDETERMINATE; ELSE THE STRUCTURE IS NONEMBEDDABLE.
C
    8 ISSUE = -1
      IF (FCTVAL/GRDNRM .LT. SMALL) ISSUE =  0
      IF (FCTVAL .LT. FCTMIN) ISSUE = 1
C
      RETURN
C
   10 FORMAT(/,' BEGIN EMBED (',16A5,')',/)
   20 FORMAT(/,' *** CAUTION: POOR INITIAL COORDINATES ***',/,
     &      5X,     'EIGENVALUES =  ',            3E13.6,   /,
     &      5X,     'RMS DEVIATION FROM BOUNDS = ',E13.6,   /)
   30 FORMAT(/,' RESULTS OF EMBED: STRCTURE  NUMBER         = ',I8)
   40 FORMAT(19X,                 'NUMBER OF ITERATIONS     = ',I8,/,
     &       19X,                 'NUMBER OF FUNCTION CALLS = ',I8,/,
     &       19X,                 'FINAL ERROR ACHIEVED     = ',E13.6,/,
     &       19X,                 'FINAL GRADIENT NORM      = ',E13.6,/)
   50 FORMAT(/,' *** CAUTION: ERROR ',2A5' IN SCCG ***',/)
   60 FORMAT(///,' EMBEDDED COORDINATES:',/)
   70 FORMAT(1X,I5,3F16.6)
      END
      SUBROUTINE CHIRIN (DUMP,NDIM,NCHIR,LIG,BND,VAL)
C
C   SETS THE VALUES OF THE CHIRIAL TRIPLE PRODUCT FOR USE BY CHIRER.
C   THIS IS EQUAL THE SQRT OF THE DETERMINANT OF THE MATRIX OF DOT
C   PRODUCTS OF THE VECTORS FROM ATOM 4 TO ATOMS 1, 2 & 3. ASSUMES
C   THAT IN EACH CHIRIAL QUARTET 1 -> 2 -> 3 COUNTERCLOCKWISE WHEN
C   ATOM 4 IS POINTING AWAY FROM YOU. NB: ALL FOUR ATOMS MUST BE AT
C   AT FIXED DISTANCES FROM EACHOTHER.
C
      LOGICAL DUMP
      DIMENSION LIG(4,NCHIR),VAL(NCHIR),BND(NDIM,NDIM)
      DOT(A,B,C) = ( A**2 + B**2 - C**2 ) / 2.
      IF (DUMP) WRITE(20,10)
      DO 1 I=1,NCHIR
      A1 = BND(LIG(1,I),LIG(4,I))**2
      B2 = BND(LIG(2,I),LIG(4,I))**2
      C3 = BND(LIG(3,I),LIG(4,I))**2
      A2 = DOT( BND(LIG(1,I),LIG(4,I)),BND(LIG(2,I),LIG(4,I)),
     &          BND(LIG(1,I),LIG(2,I)) )
      A3 = DOT( BND(LIG(1,I),LIG(4,I)),BND(LIG(3,I),LIG(4,I)),
     &          BND(LIG(1,I),LIG(3,I)) )
      B3 = DOT( BND(LIG(2,I),LIG(4,I)),BND(LIG(3,I),LIG(4,I)),
     &          BND(LIG(2,I),LIG(3,I)) )
      B1 = A2
      C1 = A3
      C2 = B3
      DET = A1*(B2*C3-B3*C2) - A2*(B1*C3-B3*C1) + A3*(B1*C2-B2*C1)
      IF (DET.LT.1.0E-4) DET = 0.0
      VAL(I) = SQRT(AMAX1(0.0,DET))
      IF (DUMP) WRITE(20,11) DET
    1 CONTINUE
      RETURN
   10 FORMAT(/,' RESULTS OF CHIRALITY INITIALIZATION:',/)
   11 FORMAT(' DETERMINANT = ',E13.6)
      END
      SUBROUTINE TRNGL (DUMP,ND,NA,LIM,B)
C
C   ITERATIVELY GO OVER ALL TRIPLES OF ATOMS, LOWERING THE UPPER
C   BOUNDS WHENEVER THESE EXCEED TRIANGLE INEQUALITY LIMIT.
C
      DIMENSION B(ND,ND)
      LOGICAL DUMP
      N = NA
      NM1 = N-1
      DO 1 L=1,LIM
      NCHG = 0
      DO 2 I=1,NM1
      IP1 = I+1
      IM1 = I-1
      DO 2 J=IP1,N
C   K < I, K < J
      IF (IM1 .EQ. 0) GO TO 4
      DO 3 K=1,IM1
      IF (B(K,I) + B(K,J) .GE. B(I,J)) GO TO 3
      B(I,J) = B(K,I) + B(K,J)
      IF(B(I,J).LT.B(J,I)) WRITE(20,9) J,I,B(J,I),K,I,B(K,I),K,J,
     1B(K,J),L
      NCHG = NCHG+1
    3 CONTINUE
    4 CONTINUE
C   K > I, K < J
      JM1 = J-1
      IF (JM1 .LT. IP1) GO TO 6
      DO 5 K=IP1,JM1
      IF (B(I,K) + B(K,J) .GE. B(I,J)) GO TO 5
      B(I,J) = B(I,K) + B(K,J)
      IF(B(I,J).LT.B(J,I)) WRITE(20,9) J,I,B(J,I),I,K,B(I,K),K,J,
     1B(K,J),L
      NCHG = NCHG+1
    5 CONTINUE
    6 CONTINUE
C   K > I, K > J
      JP1 = J+1
      IF (JP1 .GT. N) GO TO 2
      DO 7 K=JP1,N
      IF (B(I,K) + B(J,K) .GE. B(I,J)) GO TO 7
      B(I,J) = B(I,K) + B(J,K)
      IF(B(I,J).LT.B(J,I)) WRITE(20,9) J,I,B(J,I),I,K,B(I,K),J,K,
     1B(J,K),L
      NCHG = NCHG+1
    7 CONTINUE
    2 CONTINUE
      IF (NCHG .EQ. 0) RETURN
      IF (DUMP) WRITE(20,8) L, NCHG
    8 FORMAT(' ITERATION', I5, '   OF TRNGL;', I5, '   CHANGES MADE')
    1 CONTINUE
    9 FORMAT(/,' WARNING: LOWER BOUND',2I4,' = ',F8.3,' > UPPER BOUND',
     &       2I4,' = ',F8.3,' + UPPER BOUND',2I4,' = ',F8.3,
     2       ' AT ITRN ',I2,' OF TRNGL',/)
      RETURN
      END
      SUBROUTINE TRINV (DUMP,ND,NA,LIM,B)
C
C   ITERATIVELY GO OVER ALL ATOM TRIPLES, RAISING THE LOWER BOUNDS WHEN-
C   EVER THESE ARE LESS THAN POSSIBLE BY THE INVERSE TRIANGLE EQUALITY.
C   NB: MUST BE DONE ONLY AFTER USING TRNGL ON THE UPPER BOUNDS.
C
      DIMENSION B(ND,ND)
      LOGICAL DUMP
      N = NA
      NM1 = N-1
      DO 1 L=1,LIM
      NCHG = 0
      DO 2 I=1,NM1
      IP1 = I+1
      IM1 = I-1
      DO 2 J=IP1,N
C   K < I, K < J
      IF (IM1 .EQ. 0) GO TO 4
      DO 3 K=1,IM1
      IF (B(I,K) .GE. B(J,I) - B(K,J)) GO TO 33
      B(I,K) = B(J,I) - B(K,J)
      IF(B(K,I).LT.B(I,K)) WRITE(20,9) K,I,B(K,I),J,I,B(J,I),K,J,
     1B(K,J),L
      GO TO 333
   33 IF (B(J,K) .GE. B(J,I) - B(K,I)) GO TO 3
      B(J,K) = B(J,I) - B(K,I)
      IF(B(K,J).LT.B(J,K)) WRITE(20,9) K,J,B(K,J),J,I,B(J,I),K,I,
     1B(K,I),L

  333 NCHG = NCHG+1
    3 CONTINUE
    4 CONTINUE
C   K > I, K < J
      JM1 = J-1
      IF (JM1 .LT. IP1) GO TO 6
      DO 5 K=IP1,JM1
      IF (B(K,I) .GE. B(J,I) - B(K,J)) GO TO 55
      B(K,I) = B(J,I) - B(K,J)
      IF(B(I,K).LT.B(K,I)) WRITE(20,9) I,K,B(I,K),J,I,B(J,I),K,J,
     1B(K,J),L
      GO TO 555
   55 IF (B(J,K) .GE. B(J,I) - B(I,K)) GO TO 5
      B(J,K) = B(J,I) - B(I,K)
      IF(B(K,J).LT.B(J,K)) WRITE(20,9) K,J,B(K,J),J,I,B(J,I),I,K,
     1B(I,K),L
  555 NCHG = NCHG+1
    5 CONTINUE
    6 CONTINUE
C   K > I, K > J
      JP1 = J+1
      IF (JP1 .GT. N) GO TO 2
      DO 7 K=JP1,N
      IF (B(K,I) .GE. B(J,I) - B(J,K)) GO TO 77
      B(K,I) = B(J,I) - B(J,K)
      IF(B(I,K).LT.B(K,I)) WRITE(20,9) I,K,B(I,K),J,I,B(J,I),J,K,
     1B(J,K),L
      GO TO 777
   77 IF (B(K,J) .GE. B(J,I) - B(I,K)) GO TO 7
      B(K,J) = B(J,I) - B(I,K)
      IF(B(J,K).LT.B(K,J)) WRITE(20,9) J,K,B(J,K),J,I,B(J,I),I,K,
     1B(I,K),L
  777 NCHG = NCHG+1
    7 CONTINUE
    2 CONTINUE
      IF (NCHG .EQ. 0) RETURN
      IF (DUMP) WRITE(20,8) L, NCHG
    8 FORMAT(' ITERATION', I5, '   OF TRINV;', I5, '   CHANGES MADE')
    1 CONTINUE
    9 FORMAT(/,' WARNING: UPPER BOUND',2I4,' = ',F8.3,' < LOWER BOUND',
     &       2I4,' = ',F8.3,' - UPPER BOUND',2I4,' = ',F8.3,
     2       ' AT ITRN ',I2,' OF TRINV',/)
      RETURN
      END
      SUBROUTINE GRAFIC(ND,NA,BND,LABEL)
C
C   OUTPUTS LARGE MATRICES
C
      DIMENSION BND(ND,ND)
      DIMENSION LABEL(10)
      N = NA
      WRITE (20,11)
      WRITE (20,12) LABEL
      DMIN = BND(1,1)
      DMAX = BND(1,1)
      AMIN = BND(1,2)
      AMAX = BND(1,2)
      BMIN = BND(2,1)
      BMAX = BND(2,1)
      DO 1 I=2,N
      IF (BND(I,I) .GT. DMAX) DMAX = BND(I,I)
      IF (BND(I,I) .LT. DMIN) DMIN = BND(I,I)
      IM1 = I-1
      DO 1 J=1,IM1
      IF (BND(J,I) .GT. AMAX) AMAX = BND(J,I)
      IF (BND(J,I) .LT. AMIN) AMIN = BND(J,I)
      IF (BND(I,J) .GT. BMAX) BMAX = BND(I,J)
      IF (BND(I,J) .LT. BMIN) BMIN = BND(I,J)
    1 CONTINUE
      RCL = (BMAX-BMIN)/5.
      SCL = (AMAX-AMIN)/5.
      TCL = (DMAX-DMIN)/10.
      IF (RCL .EQ. 0) RCL = 1.
      IF (SCL .EQ. 0) SCL = 1.
      IF (TCL .EQ. 0) TCL = 1.
      WRITE (20,2) AMIN, AMAX
      WRITE (20,3) DMIN, DMAX
      WRITE (20,4) BMIN, BMAX
    2 FORMAT(' ABOVE DIAG ELTS RANGE FROM', F9.3, '   TO', F9.3)
    3 FORMAT(' DIAGONAL  ELTS  RANGE FROM', F9.3, '   TO', F9.3)
    4 FORMAT(' BELOW DIAG ELTS RANGE FROM', F9.3, '   TO', F9.3,/)
      CALL MATPRT (BND,ND,NA)
      WRITE (20,11)
   11 FORMAT(/,' ---------------------------------------------------',/)
   12 FORMAT(1X,10A4,':',/)
      RETURN
      END
      SUBROUTINE MATPRT (A,NDIM,N)
C
C  THIS ROUTINE PRINTS A REAL ARRAY IN A DECENT FORMAT
C
      DIMENSION A(NDIM,NDIM)
C
      LIM1 = 1
   10 LIM2 = LIM1 + 11
      IF (LIM2.GT.N) LIM2 = N
      WRITE(20,1010) (K,K=LIM1,LIM2)
 1010 FORMAT (//9X,11(I3,6X),I3)
      DO 20 I = 1,N
      WRITE (20,1020) I,(A(I,J),J=LIM1,LIM2)
 1020 FORMAT (1X,I3,2X,12F9.5)
   20 CONTINUE
      IF (LIM2.EQ.N) RETURN
      LIM1 = LIM1 + 12
      GO TO 10
      END
      SUBROUTINE XDSHOW (DIST,XYZ,ND,NA)
      DIMENSION DIST(ND,ND), XYZ(3,72)
C
      WRITE(20,10)
   10 FORMAT (/,' CURRENT COORDINATES ARE:')
      DO 100 I = 1,NA
      WRITE(20,20) I,XYZ(1,I),XYZ(2,I),XYZ(3,I)
   20 FORMAT (5X,I2,3F15.6)
  100 CONTINUE
      WRITE (20,30)
   30 FORMAT (/,' AND THE CURRENT DISTANCE MATRIX IS:')
      DO 200 I = 1,NA
      DO 200 J = 1,I
      DIST(I,J) = SQRT((XYZ(1,I)-XYZ(1,J))**2 + (XYZ(2,I)-XYZ(2,J))**2
     1 + (XYZ(3,I)-XYZ(3,J))**2)
      DIST(J,I) = DIST(I,J)
  200 CONTINUE
      CALL MATPRT (DIST,ND,NA)
      RETURN
      END
      SUBROUTINE DSTMAT (NDIM,NPNTS,DMX,BND)
C
C   COMPUTES A MATRIX OF TRIANGLE-CORRELATED RANDOM FRACTIONS OF THE
C   DISTANCES BETWEEN THE UPPER AND LOWER BOUNDS (SEE CRIPPEN, IN PRESS)
C
      DIMENSION DMX(NDIM,NDIM),BND(NDIM,NDIM)
      N = NPNTS
      NM1 = N-1
C
C   GENERATE THE RANDOM NUMBERS
C
      DO 20 I=1,NM1
      JSTART=I+1
      DMX(I,I)=0.
      DO 10 J=JSTART,N
      DMX(J,I)=RAN(0)
  10  CONTINUE
  20  CONTINUE
      DMX(N,N)=0.
C
C   CALCULATE DISTANCE CORRELATIONS
C
      DO 100 I=1,NM1
      JSTART=I+1
      DO 90 J=JSTART,N
      DENOM=0.
      DMX(I,J)=0.
      DO 80 K=1,N
      MIK = MAX0(I,K)
      MJK = MAX0(J,K)
      NIK = MIN0(I,K)
      NJK = MIN0(J,K)
      IF(K.EQ.I) GO TO 80
      IF(K.NE.J) GO TO 40
C
C   SELF CORRELATION IS 1.
C
      DMX(I,J)=DMX(I,J)+DMX(J,I)
      DENOM=DENOM+1.
      GO TO 80
C
C   POSITIVE CORRELATION CASES
C
  40  IF(BND(NJK,MJK).GT.0.2*BND(NIK,MIK)) GO TO 50
      IF(I.GT.K) CORR=0.9*DMX(I,K)
      IF(K.GT.I) CORR=0.9*DMX(K,I)
      DMX(I,J)=DMX(I,J)+CORR
      DENOM=DENOM+0.9
      GO TO 80
  50  IF(BND(NIK,MIK).GT.0.2*BND(NJK,MJK)) GO TO 60
      IF(J.GT.K) CORR=0.9*DMX(J,K)
      IF(K.GT.J) CORR=0.9*DMX(K,J)
      DMX(I,J)=DMX(I,J)+CORR
      DENOM=DENOM+0.9
      GO TO 80
C
C   NEGATIVELY CORRELATED DISTANCES
C
  60  IF(BND(MIK,NIK).LT.0.9*BND(NJK,MJK)) GO TO 70
      IF(J.GT.K) CORR=0.5*(1.-DMX(J,K))
      IF(K.GT.J) CORR=0.5*(1.-DMX(K,J))
      DMX(I,J)=DMX(I,J)+CORR
      DENOM=DENOM+0.5
      GO TO 80
  70  IF(BND(MJK,NJK).LT.0.9*BND(NIK,MIK)) GO TO 80
      IF(I.GT.K) CORR=0.5*(1.-DMX(I,K))
      IF(K.GT.I) CORR=0.5*(1.-DMX(K,I))
      DMX(I,J)=DMX(I,J)+CORR
      DENOM=DENOM+0.5
  80  CONTINUE
  90  DMX(I,J)=DMX(I,J)/DENOM
 100  CONTINUE
C
C   SCALE WITHIN BOUNDS TO GET TRIAL DISTANCE MATRIX
C
      DO 200 I=1,NM1
      IP1 = I+1
      DO 200 J=IP1,N
      DMX(I,J) = BND(J,I) + (BND(I,J)-BND(J,I)) * DMX(I,J)
      DMX(J,I) = DMX(I,J)
  200 CONTINUE
      RETURN
      END
      SUBROUTINE METRIC (DUMP,NDIM,NPNTS,DSQ,GMX,NNEG)
C
C   COMPUTES THE METRIC OR GRAM MATRIX OF ALL POSSIBLE DOT PRODUCTS
C   BETWEEN THE ATOMIC VECTORS TO THE CENTER OF MASS USING THE LAW
C   OF COSINES AND THE FOLLOWING FORMULA FOR THE DISTANCES TO THE
C   CENTER OF MASS: DCM(I)**2 = (1/N) * SUM(J=1,N)(DST(I,J)**2) -
C                               (1/N**2) * SUM(J<K)(DST(J,K)**2)
C
      LOGICAL DUMP
      DIMENSION DSQ(NDIM),GMX(NDIM,NDIM)
      N = NPNTS
      NM1 = N-1
C
      DSUMJK = 0.
      DO 10 I=1,NM1
      IP1=I+1
      DO 10 J=IP1,N
C
C   SQUARE THE DISTANCE MATRIX AND TAKE SUM OVER UPPER TRIANGLE
C
      GMX(I,J) = GMX(I,J)**2
      GMX(J,I) = GMX(I,J)
      DSUMJK = DSUMJK + GMX(I,J)
 10   CONTINUE
C
C   DIVIDE BY N SQUARED FOR CENTER OF MASS CALCULATIONS
C
      DSUMJK = DSUMJK / FLOAT(N*N)
      NNEG = 0
      DO 30 I=1,N
C
C   SUM SQUARED DISTANCES FROM ATOM I
C
      DSUMJ = 0.
      DO 20 J=1,N
      DSUMJ = DSUMJ + GMX(I,J)
 20   CONTINUE
C
C   CENTER OF MASS DERIVED USING THE ABOVE FORMULA
C
      DSQ(I) = DSUMJ / FLOAT(N) - DSUMJK
      IF (DSQ(I) .LT. 0.) NNEG = NNEG+1
 30   CONTINUE
C
C   CALCULATION OF METRIC MATRIX USING THE LAW OF COSINES
C
      DO 40 I=1,NM1
      IP1 = I+1
      GMX(I,I) = DSQ(I)
      DO 40 J=IP1,N
      GMX(I,J) = (DSQ(I) + DSQ(J) - GMX(I,J)) / 2.
      GMX(J,I) = GMX(I,J)
 40   CONTINUE
      GMX(N,N) = DSQ(N)
C
      IF (DUMP) WRITE (20,50) DSUMJK
      IF (DUMP) WRITE (20,60) (DSQ(I),I=1,N)
C
      RETURN
 50   FORMAT(/,' RESULTS OF METRIC:  SQUARED RADIUS OF GYRATION = ',
     & F13.6,/,' SQUARED DISTANCES TO THE CENTER OF MASS:',/)
 60   FORMAT(10F13.6)
      END
      SUBROUTINE EIGENV (DUMP,NDIM,NPNTS,LAM,LIM,RAT,EVL,EVC,DWK,EWK,OK)
C
C   COMPUTES THE FIRST LAM LARGEST MAGNITUDE EIGENVALUES/EIGENVECTORS OF
C   A REAL SYMMETRIC MATRIX BY THE POWER METHOD; OK = .TRUE. IMPLIES
C   THAT THE EIGENVALUES CAME OUT POSITIVE AND IN ORDER.
C
      LOGICAL OK,DUMP
      DIMENSION EVL(3),EVC(3,NDIM),DWK(NDIM),EWK(NDIM,NDIM)
      INTEGER FMT(2)
      N = NPNTS
      OK = .TRUE.
      ENCODE(10,1,FMT) LAM
    1 FORMAT(1H(,I1,6HE15.6))
C
      DO 333 M=1,LAM
      EVL(M) = 0.
C
C   1ST MATRIX MULTIPLICATION; INITIAL GUESS IS OLD COORDINATES
C
      DOT1=0.
      DO 101 I=1,N
      DWK(I)=0.
      DO 100 J=1,N
      DWK(I) = DWK(I) + EWK(I,J)*EVC(M,J)
  100 CONTINUE
      DOT1 = DOT1 + DWK(I)**2
  101 CONTINUE
C
C   IF IN OR NEAR NULL SPACE, USE A RANDOM GUESS AT FIRST VECTOR
C
      IF (DOT1.GT.FLOAT(N)/10000.) GO TO 111
      DO 110 I=1,N
      DWK(I) = RAN(0)
  110 CONTINUE
C
  111 ITOT = 0
      DO 300 ITER=1,LIM
C
C   MULTIPLY VECTOR BY MATRIX AND COMPUTE DOT PRODUCTS
C
      DOT1=0.
      DOT2=0.
      DO 202 I=1,N
      EVC(M,I)=0.
      DO 200 J=1,N
      EVC(M,I) = EVC(M,I) + EWK(I,J)*DWK(J)
  200 CONTINUE
      DOT1 = DOT1 + EVC(M,I)**2
      DOT2 = DOT2 + EVC(M,I)*DWK(I)
  202 CONTINUE
C
C   NORMALIZE NEW VECTOR AND SUBSTITUTE FOR OLD
C
      RATIO = ABS((EVL(M)-DOT2)/DOT2)
      EVL(M) = DOT2
      DOT1 = SQRT(DOT1)
      DO 220 I=1,N
      EVC(M,I) = EVC(M,I) / DOT1
      DWK(I) = EVC(M,I)
  220 CONTINUE
C
C   WHEN DOT OF NEW WITH OLD HAS CONVERGED TO EIGENVALUE WITH ACCURACY
C   WITHIN RAT, GO TO COMPUTATION OF NEXT EIGENVALUE/EIGENVECTOR.
C
      IF (RATIO.LT.RAT) GO TO 303
C
  300 CONTINUE
      OK = .FALSE.
C
C   ELIMINATE OLD EIGENVALUE FROM MATRIX
C
  303 ITOT = ITOT + ITER
      DO 330 I=1,N
      DO 330 J=1,N
      EWK(I,J) = EWK(I,J) - EVL(M)*EVC(M,I)*EVC(M,J)
  330 CONTINUE
C
      IF (EVL(M).LT.0.) OK = .FALSE.
  333 CONTINUE
C
      IF (ABS(EVL(1)).LT.ABS(EVL(2))) OK = .FALSE.
      IF (ABS(EVL(2)).LT.ABS(EVL(3))) OK = .FALSE.
C
      IF (DUMP) WRITE (20,1000) ITOT
      IF (DUMP) WRITE (20,FMT)  (EVL(I),I=1,LAM)
      IF (DUMP) WRITE (20,1001)
      IF (DUMP) WRITE (20,FMT) ((EVC(I,J),I=1,LAM),J=1,N)
      RETURN
 1000 FORMAT(/,' RESULTS OF EIGENV: ',I5,' ITERATIONS TOTAL',//,
     &         ' EIGENVALUES:')
 1001 FORMAT(/,' EIGENVECTORS:')
      END
      SUBROUTINE COORDS (DUMP,OK,NDIM,NATOM,NPNTS,IPS,BND,
     &                   EVLA,EVLB,XYZA,XYZB,RMSB)
C
C   CONVERTS EIGENVALUES/VECTORS INTO COORDINATES AND CALLS RMSERS TO
C   COMPUTE THE RMS DEVIATION FROM THE BOUNDS; IF THIS VALUE IS THE
C   LOWEST SO FAR OR THE EIGENVALUES ARE POSITIVE & IN ORDER, IT SAVES
C   THESE COORDINATES FOR USE IN OPTIMIZATION, AND SQUARES THE BOUNDS
C   USE BY ERRFCT DURING OPTIMIZATION.
C
      LOGICAL OK,DUMP
      DIMENSION EVLA(3),EVLB(3),XYZA(3,NDIM),XYZB(3,NDIM),BND(NDIM,NDIM)
      DIMENSION LABEL(5)
      DATA (LABEL(I),I=1,5)  /4H BEF,4HORE ,4HOPTI,4HMIZA,4HTION/
      NA = NATOM
      NP = NPNTS
      NPM1 = NP-1
C
C   COMPUTE COORDINATES: THERE IS NO JUSTIFICATION FOR USING THE ABS-
C   OLUTE VALUE OF NEGATIVE EIGENVALUES EXCEPT THAT THERE IS NOTHING
C   ELSE WE CAN DO AND IN PRACTICE IT GENERALLY EVENTUALLY CONVERGES.
C
      DO 1 I=1,3
      ROOT = SQRT(ABS(EVLA(I)))
      DO 1 J=1,NA
      XYZA(I,J) = ROOT * XYZA(I,J)
    1 CONTINUE
C
      CALL RMSERS (DUMP,.FALSE.,NDIM,NPNTS,XYZA,BND,LABEL,
     &             RMSA,HI,IHI,JHI,HO,IHO,JHO)
C
C   SAVE THE COORDINATES IF THEY'RE 'OK' OR HAVE THE LOWEST RMS SO FAR.
C
      IF (.NOT.OK .AND. RMSA.GT.RMSB) GO TO 3
      DO 2 I=1,3
      EVLB(I) = EVLA(I)
      DO 2 J=1,NA
      XYZB(I,J) = XYZA(I,J)
    2 CONTINUE
      RMSB = RMSA
C
C   SQUARE THE BOUNDS FOR USE BY ERRFCT IF WE ARE READY TO GO ON.
C
    3 IF (.NOT.OK .AND. IPS.GT.0) RETURN
      DO 4 J=1,NPM1
      JP1 = J+1
      DO 4 K=JP1,NP
      BND(J,K) = BND(J,K)**2
      BND(K,J) = BND(K,J)**2
    4 CONTINUE
      RETURN
      END
      SUBROUTINE RMSERS (DUMP,SQUARE,NDIM,NPNTS,XYZ,BND,LABEL,
     &                   RMSP,HIP,IHIP,JHIP,HOP,IHOP,JHOP)
C
C   COMPUTES THE RMS DEVIATION FROM THE BOUNDS AS WELL AS THE MAXIMUM
C   ABSOLUTE DEVIATIONS AND POSITIONS THEREOF FOR USE IN EVALUATING CON-
C   VERGENCE. IF SQUARE = .TRUE. THE BOUNDS ARE ASSUMED TO BE SQUARED.
C
      LOGICAL DUMP,SQUARE
      DIMENSION LABEL(5),XYZ(3,NDIM),BND(NDIM,NDIM)
      REAL LOBD,UPBD
      NM1 = NPNTS-1
      RMS = 0.
      HIMX = 0.
      HOMX = 0.
      DO 1 I=1,NM1
      IP1 = I+1
      DO 1 J=IP1,NPNTS
      UPBD = BND(I,J)
      LOBD = BND(J,I)
      IF (SQUARE) UPBD = SQRT(UPBD)
      IF (SQUARE) LOBD = SQRT(LOBD)
      DSTSQ = (XYZ(1,I)-XYZ(1,J))**2 + (XYZ(2,I)-XYZ(2,J))**2 +
     &                 (XYZ(3,I)-XYZ(3,J))**2
      DST = SQRT(DSTSQ)
      HI = DST-UPBD
      HO = LOBD-DST
      IF (HI.GT.0.) RMS = RMS + HI**2
      IF (HO.GT.0.) RMS = RMS + HO**2
      IF (.NOT.DUMP) GO TO 1
      IF (HI.GT.HIMX) IHI = I
      IF (HI.GT.HIMX) JHI = J
      IF (HO.GT.HOMX) IHO = I
      IF (HO.GT.HOMX) JHO = J
      HIMX = AMAX1(HI,HIMX)
      HOMX = AMAX1(HO,HOMX)
    1 CONTINUE
      NPAIR = NPNTS*(NPNTS-1)/2
      RMS = SQRT(RMS/FLOAT(NPAIR))
      IF (DUMP) WRITE (20,11) LABEL,HOMX,IHO,JHO,HIMX,IHI,JHI,RMS
      RMSP = RMS
      HIP  = HIMX
      IHIP = IHI
      JHIP = JHI
      HOP  = HOMX
      IHOP = IHO
      JHOP = JHO
      RETURN
   11 FORMAT(/,' FIT TO BOUNDS ',5A4,':',
     &         ' MAX LOWER BOUND VIOLATION = ',F12.6,' OCCURS AT ',2I5,
     &   /,36X,' MAX UPPER BOUND VIOLATION = ',F12.6,' OCCURS AT ',2I5,
     &   /,36X,' RMS DEVIATION FROM BOUNDS = ',F12.6,/)
      END
      SUBROUTINE DMDUMP (NDIM,NPNTS,XYZ,BND,DMD)
C
C   GENERATES THE DISTANCE MATRIX OF THE FINAL STRUCTURE IN UPPER HALF
C   OF DMD AND THE INDIVIDUAL TERMS OF THE FINAL ERROR IN THE LOWER.
C
      DIMENSION LABEL(10),XYZ(3,NDIM),BND(NDIM,NDIM),DMD(NDIM,NDIM)
      DATA (LABEL(I),I=1,10) /4HFINA,4HL DI,4HST M,4HAT A,4HBV; ,
     &                        4H DCM,4H DIA,4HG; E,4HRROR,4H BEL/
      N = NPNTS
      DO 1 I=1,N
      DMD(I,I) = 0.
    1 CONTINUE
C
      NM1 = N-1
      DSQSUM = 0.
      DO 2 I=1,NM1
      IP1 = I+1
      DO 2 J=IP1,N
      DSTSQ = (XYZ(1,I)-XYZ(1,J))**2 + (XYZ(2,I)-XYZ(2,J))**2
     &                  + (XYZ(3,I)-XYZ(3,J))**2
      DSQSUM = DSQSUM + DSTSQ
      DMD(I,I) = DMD(I,I) + DSTSQ
      DMD(J,J) = DMD(J,J) + DSTSQ
      DST = SQRT(DSTSQ)
      DMD(I,J) = DST
      DMD(J,I) = 0.
      IF (DSTSQ.GT.BND(I,J)) DMD(J,I) = ((DSTSQ-BND(I,J)) / BND(I,J))**2
      IF (DSTSQ.LT.BND(J,I)) DMD(J,I) = ((BND(J,I)-DSTSQ) / BND(J,I))**2
    2 CONTINUE
C
      FN = FLOAT(N)
      ROFGSQ = DSQSUM/FN**2
      DO 3 I=1,N
      DMD(I,I) = SQRT(DMD(I,I)/FN - ROFGSQ)
    3 CONTINUE
C
      CALL GRAFIC (NDIM,N,DMD,LABEL)
C
      ROFGS1 = SQRT(ROFGSQ)
      WRITE(20,10) ROFGS1
C
      RETURN
   10 FORMAT(/,' RADIUS OF GYRATION = ',E13.6,/)
      END
      FUNCTION ERRFCT (NATOM,XYZ,GRD,NOGRAD)
C
C   ERROR FUNCTION FOR EMBEDDING WHICH GOES TO ZERO IFF THE BOUNDS
C   ARE SATISFIED, AND GRADIENT THEREOF WRT CARTESIAN COORDINATES.
C   IN THIS VERSION WE DIVIDE EACH TERM BY THE BOUND VIOLATED SO
C   THAT WE EXERT AN EQUAL 'PULL' ON SHORT AND LONG DISTANCES.
C   NB: VARIABLY DIMENSIONING WITH NATOM WILL RESULT IN AN ARRAY
C   SUBSCRIPT OUT OF BOUNDS IF WE HAVE OUTRIGGERS, BUT SINCE THE
C   ARRAYS ARE CONTIGUOUS IN MEMORY THIS WILL BE ALL RIGHT.
C
      COMMON /BNDS/ NPNTS,BND(72,72)
      DIMENSION GRD(3,NATOM),XYZ(3,NATOM)
      LOGICAL NOGRAD
      NA = NATOM
      NP = NPNTS
      NPM1 = NP-1
C
C   COMPUTE CHIRALITY CONTRIBUTIONS TO ERROR AND GRADIENT.
C
      ERROR = CHIRER (NA,XYZ,GRD,NOGRAD)
C
      DO 100 I=1,NPM1
      IP1 = I+1
      DO 100 J=IP1,NP
C
C   CALCULATE DELTAS ALONG THE GIVEN AXES.
C
      DX = XYZ(1,I) - XYZ(1,J)
      DY = XYZ(2,I) - XYZ(2,J)
      DZ = XYZ(3,I) - XYZ(3,J)
C
C   CALCULATE SQUARED DISTANCES AND BOUNDS.
C
      DSTSQ = DX*DX + DY*DY + DZ*DZ
      BUPSQ = BND(I,J)
      BLOSQ = BND(J,I)
C
C   UPPER BOUND VIOLATION TEST.
C
      IF (DSTSQ.LE.BUPSQ) GO TO 10
      RAWER = (DSTSQ-BUPSQ) / BUPSQ
      CHAIN = 4.0 * RAWER / BUPSQ
      GO TO 20
C
C   LOWER BOUND VIOLATION TEST.
C
  10  IF (DSTSQ.GE.BLOSQ) GO TO 100
      RAWER = (DSTSQ-BLOSQ) / BLOSQ
      CHAIN = 4.0 * RAWER / BLOSQ
C
C   SUM UP ERRORS OVER ALL DISTINCT PAIRS.
C
  20  ERROR = ERROR + RAWER**2
C
C   IF NOGRAD, WE DO NOT COMPUTE THE GRADIENT.
C
      IF (NOGRAD) GO TO 100
C
C   CHAIN--COMPOUND FACTOR IS PARTIAL DERIVATIVE.
C
      GX = DX * CHAIN
      GY = DY * CHAIN
      GZ = DZ * CHAIN
C
C   ADD TO GRADIENT OF NONOUTRIGGERS.
C
      IF (I.GT.NA) GO TO 30
      GRD(1,I) =  GX + GRD(1,I)
      GRD(2,I) =  GY + GRD(2,I)
      GRD(3,I) =  GZ + GRD(3,I)
C
C   SUBTRACT FROM GRADIENT OF NONOUTRIGGERS.
C
  30  IF (J.GT.NA) GO TO 100
      GRD(1,J) = -GX + GRD(1,J)
      GRD(2,J) = -GY + GRD(2,J)
      GRD(3,J) = -GZ + GRD(3,J)
C
 100  CONTINUE
C
      ERRFCT = ERROR
      RETURN
      END
      FUNCTION CHIRER (NATOM,XYZ,GRD,NOGRAD)
C
C   COMPUTES THE ORIENTED VOLUME OF THE PARALLELEPIPED SPANNED BY THE
C   VECTORS FROM THE ASYMMETRIC CARBON TO THREE OF ITS LIGANDS, AND
C   SQUARES THE DIFFERENCE BETWEEN THIS AND THE ORIENTED VOLUME WHEN
C   THE CHIRALITY AND BOND ANGLES ARE CORRECT. RETURNS THE SUM OF THIS
C   QUANTITY OVER ALL CHIRAL CENTERS ALONG WITH THE CORRESPONDING
C   GRADIENT UNLESS NOGRAD.
C
      LOGICAL NOGRAD
      COMMON /CHIR/ NCHIR,LIG(4,120),VAL(120)
      DIMENSION XYZ(3,NATOM),GRD(3,NATOM)
      DIMENSION ABC(3,3)
      NA = NATOM
C
C   ZERO GRADIENT AND ERROR FOR CHIRER AND ERRFCT.
C
      CHIRER=0.
      IF (NOGRAD) GO TO 2
      DO 1 I=1,3
      DO 1 J=1,NA
      GRD(I,J) = 0.
    1 CONTINUE
    2 IF(NCHIR.LE.0) RETURN
C
      DO 100 M=1,NCHIR
C
C   COMPUTE MATRIX OF RELATIVE COORDINATES, TAKE ITS DETERMINANT AND
C   SUM SQUARES OF DIFFERENCES WITH CORRECT VALUES OVER ALL CHIRALS.
C
      DO 10  I=1,3
      DO 10  J=1,3
      ABC(I,J) = XYZ(I,LIG(J,M)) - XYZ(I,LIG(4,M))
   10 CONTINUE
      COFT1 = ABC(2,2)*ABC(3,3)-ABC(2,3)*ABC(3,2)
      COFT2 = ABC(2,3)*ABC(3,1)-ABC(2,1)*ABC(3,3)
      COFT3 = ABC(2,1)*ABC(3,2)-ABC(2,2)*ABC(3,1)
      VOL = ABC(1,1)*COFT1 + ABC(1,2)*COFT2 + ABC(1,3)*COFT3
      CHIRER = CHIRER + (VOL-VAL(M))**2
      IF (NOGRAD) GO TO 100
C
C   DERIVATIVE OF  CHIRER  EQUALS  2 X DIFFERENCE  X  COFACTER OF MATRIX
C   ABC  WRT ELEMENT CONTAINING COORDINATE  X  SIGN OF COORDINATE SUMMED.
C
      GP = 2.0 * (VOL-VAL(M))
      GQ = ABC(2,2)*ABC(3,3)-ABC(3,2)*ABC(2,3)
      GRD(1,LIG(1,M)) = GRD(1,LIG(1,M)) + GP*GQ
      GRD(1,LIG(4,M)) = GRD(1,LIG(4,M)) - GP*GQ
      GQ = ABC(3,2)*ABC(1,3)-ABC(1,2)*ABC(3,3)
      GRD(2,LIG(1,M)) = GRD(2,LIG(1,M)) + GP*GQ
      GRD(2,LIG(4,M)) = GRD(2,LIG(4,M)) - GP*GQ
      GQ = ABC(1,2)*ABC(2,3)-ABC(2,2)*ABC(1,3)
      GRD(3,LIG(1,M)) = GRD(3,LIG(1,M)) + GP*GQ
      GRD(3,LIG(4,M)) = GRD(3,LIG(4,M)) - GP*GQ
      GQ = ABC(3,1)*ABC(2,3)-ABC(2,1)*ABC(3,3)
      GRD(1,LIG(2,M)) = GRD(1,LIG(2,M)) + GP*GQ
      GRD(1,LIG(4,M)) = GRD(1,LIG(4,M)) - GP*GQ
      GQ = ABC(1,1)*ABC(3,3)-ABC(3,1)*ABC(1,3)
      GRD(2,LIG(2,M)) = GRD(2,LIG(2,M)) + GP*GQ
      GRD(2,LIG(4,M)) = GRD(2,LIG(4,M)) - GP*GQ
      GQ = ABC(2,1)*ABC(1,3)-ABC(1,1)*ABC(2,3)
      GRD(3,LIG(2,M)) = GRD(3,LIG(2,M)) + GP*GQ
      GRD(3,LIG(4,M)) = GRD(3,LIG(4,M)) - GP*GQ
      GQ = ABC(2,1)*ABC(3,2)-ABC(3,1)*ABC(2,2)
      GRD(1,LIG(3,M)) = GRD(1,LIG(3,M)) + GP*GQ
      GRD(1,LIG(4,M)) = GRD(1,LIG(4,M)) - GP*GQ
      GQ = ABC(3,1)*ABC(1,2)-ABC(1,1)*ABC(3,2)
      GRD(2,LIG(3,M)) = GRD(2,LIG(3,M)) + GP*GQ
      GRD(2,LIG(4,M)) = GRD(2,LIG(4,M)) - GP*GQ
      GQ = ABC(1,1)*ABC(2,2)-ABC(2,1)*ABC(1,2)
      GRD(3,LIG(3,M)) = GRD(3,LIG(3,M)) + GP*GQ
      GRD(3,LIG(4,M)) = GRD(3,LIG(4,M)) - GP*GQ
  100 CONTINUE
      RETURN
      END
