      SUBROUTINE SCCG (FCALC,NR,X,NFCT,NITN,FCTVAL,GRDNRM,STATUS,DUMP)
C
C   ****************************************************************
C   HAVEL'S IMPLEMENTATION OF THE SELF-CORRECTING CONJUGATE GRADIENT
C   OPTIMIZER OF A. PERRY (INTNL. J. COMPUTER MATH.(1978) B6 327-33)
C   ****************************************************************
C
C   ****************************************************************
C	MODIFIED FOR DEC-10 BY D. SMITH, 10/15/80
C   ****************************************************************
C
      COMMON /CTRL/ SLFCOR,IFREQ,ITNLIM,FCTMIN,GRDMIN
      COMMON /PARA/ GESFCT,STPMIN,STPMAX,CAPPA,DELTA,EPSLN,
     1              FAST,SLOW,ANGMAX,INTMAX,RESTAR
      COMMON /SCCGC/ P(3,72),Q(3,72),C(3,72),G(3,72)
      DIMENSION X(3,1)
C
C   PARAMETERS: NR     - NO. OF RESIDUES
C               FCALC  - NAME OF THE FUNCTION TO BE MINIMIZED
C               SLFCOR - SCCG IF TRUE; HSCG IF FALSE
C               GESFCT - MIN. FRACTION OF OLD FCTN DECRESE EXPECTED
C               STPMIN - MINIMUM ALLOWED LINE SEARCH STEP SIZE
C               STPMAX - MAXIMUM ALLOWED LINE SEARCH STEP SIZE
C               CAPPA  - 0 < ACCUARCY OF LINE SEARCH CONTROL < 1
C               DELTA  - ERROR IF RELATIVE FUNCTION INCREASE EXCEEDS
C               EPSLN  - ERROR IF TOTAL MOVE IS LESS THAN THIS
C               FAST   - STEEPEST DESCENT WHILE FCTN DECREASE EXCEEDS
C               SLOW   - RESTART IF RELATIVE FCTN DECREASE DROPS BELOW
C               ANGMAX - RESTART IF ANGLE BTN GRAD & CONJ GRAD EXCEEDS
C               INTMAX - MAXIMUM NO. OF INTERPOLATIONS IN LINE SEARCH
C               RESTAR - PERIODIC RESTART EVERY RESTAR ITERATIONS
C               FCTMIN - NORMAL EXIT IF FUNCTION GETS LESS THAN THIS
C               GRDMIN - NORMAL EXIT IF GRADIENT GETS LESS THAN THIS
C               ITNLIM - ERROR RETURN IF NO. OF ITERATIONS EXCEEDS THIS
C               IFREQ  - IF NONZERO, OUTPUT EVERY IFREQ'D ITERATION
C                        IF NEGATIVE, OUTPUT INCLUDES PRESENT COORDS.
C               NFCT   - NO. OF FUNCTION CALLS
C               NITN   - NO. OF ITERATIONS
C               STATUS - CHARACTER STRING CONTAINING ERROR DESCRIPTION
C               FCTVAL - FINAL FUNCTION VALUE
C               GRDNRM - FINAL GRADIENT NORM
C               X & G  - COORDINATES AND GRADIENT
C                 C    - CONJUGATE GRADIENT AND ESTIMATED PRIOR GRADIENT
C               P & Q  - DIFFERENCES BETWEEN CURRENT AND ESTIMATED CO-
C                        CORDINATES AND GRADIENTS; ALSO CONTAIN OLD CO-
C                        CORDINATES AND GRADIENTS PART OF THE TIME
C
      EXTERNAL FCALC,COEFCT,SEARCH
      LOGICAL  SLFCOR, DUMP
      INTEGER  BLANK,STATUS(2),RESTAR
      DATA BLANK /5H     /
      STATUS(1) = BLANK
      STATUS(2) = BLANK
      N = NR
      NITN = 0
      NFCT = 0
C
      IF (DUMP.AND.IFREQ.NE.0) WRITE(20,1) GESFCT,STPMIN,STPMAX,CAPPA,
     1  DELTA,EPSLN,FAST,SLOW,ANGMAX,INTMAX,RESTAR,FCTMIN,GRDMIN,
     2                   ITNLIM,SLFCOR
    1 FORMAT(/,' ENTER SCCG: PARAMETERS:',/,'  GESFCT  STPMIN  STPMAX   
     1CAPPA   DELTA   EPSLN    FAST    SLOW  ANGMAX  INTMAX  RESTAR  FCT
     2MIN  GRDMIN  ITNLIM  SLFCOR',/,9E8.2,2I8,2E8.2,I8,L8,/)
C
      NFCT = NFCT+1
      FCT = FCALC (N,X,G,.FALSE.)
      OLDFCT = 2.*FCT
      GDG = 0.
      DO 2 I=1,3
      DO 2 J=1,N
      GDG = GDG + G(I,J)**2
      P(I,J) = 0.
      Q(I,J) = 0.
      C(I,J) = 0.
    2 CONTINUE
      GNM = SQRT(GDG)
C
      IF (.NOT.DUMP) GO TO 10
      IF (IFREQ.NE.0) WRITE(20,3) FCT,GNM
    3 FORMAT(' BEGIN SCCG WITH ',18X,E9.3,' FCT VAL; ',E9.3,
     1' GRAD NORM',/)
      IF (IFREQ.GE.0) GO TO 10
      WRITE(20,4)
    4 FORMAT(' INITIAL COORDINATES:',/)
      WRITE(20,5) (J,(X(I,J),I=1,3),J=1,N)
    5 FORMAT(3(I5,1X,3F12.3))
      WRITE(20,6)
    6 FORMAT(/)
C
C    *****************************************************************
C    BEGIN: SET COEF TO ZERO AND GO TO CONJUGATE GRADIENT CALCULATION
C    *****************************************************************
C    THIS (RE)STARTS THE PROCEDURE AND IS DONE AT LEAST EVERY N ITER-
C    ATIONS. IN NORMAL RESTARTS, WE PROCEED WITH LATEST COORDINATES;
C    IN ERROR RESTARTS, WE RESTORE THE OLD COORDINATES.
C
   10 COEF = 0.
C
C   *****************************************************************
C   MAIN ITERATION LOOP FOR SELF-CORRECTING CONJUGATE GRADIENT SEARCH
C   *****************************************************************
C   IF ITN=1, WE ARE GOING IN STEEPEST DECENT DIRECTION
C
      DO 1000 ITN = 1, RESTAR
      STATUS(1) = BLANK
      STATUS(2) = BLANK
      IF (ITN.EQ.1) GO TO 100
C
C   DETERMINE THE COEFICIENT FOR COMPUTING THE NEW CONJUGATE DIRECTION
C
      COEF = COEFCT (FCALC,N,X,FCT,GDG,NFCT,STATUS,SLFCOR)
C
C   COMPUTE THE NEW SEARCH VECTOR CONJUGATE TO THE (CORRECTED) PRIOR
C   SEARCH VECTOR, AND STORE CURRENT COORDINATES AND GRADIENT
C
  100 CDC = 0.
      DO 101 I=1,3
      DO 101 J=1,N
      C(I,J) = - G(I,J) + COEF * P(I,J)
      CDC = CDC + C(I,J)**2
      P(I,J) = X(I,J)
      Q(I,J) = G(I,J)
  101 CONTINUE
      CNM = SQRT(CDC)
C
C   NORMALIZE THE THE RESULTANT VECTOR AND CHECK THE NEW SEARCH
C   DIRECTION TO MAKE SURE IT IS NOT TOO FAR FROM THE GRADIENT
C
      SLOPE = 0.
      DO 110 I=1,3
      DO 110 J=1,N
      C(I,J) = C(I,J) / CNM
      SLOPE = SLOPE + G(I,J) * C(I,J)
  110 CONTINUE
C
      COSANG = -SLOPE/GNM
      IF (COSANG.GT.1.0) COSANG = 1.0
      ANGLE =  57.2958 * ACOS(COSANG)
      IF (ANGLE.LT.ANGMAX) GO TO 111
      STATUS(1) = 5H WIDE
      STATUS(2) = 5HANGLE
      GO TO 303
C
C   PERFORM LINE SEARCH ALONG THE NEW CONJUGATE DIRECTION  C  
C
  111 FCTLOW = SEARCH (FCALC,NFCT,N,X,FCT,OLDFCT,SLOPE,STATUS)
      IF ((STATUS(1).EQ.5H INTP).AND.(STATUS(2).EQ.5HLNERR)) GO TO 2000
C
C   CHECK THAT THE FUNCTION DID NOT INCREASE SIGNIFICANTLY AND
C   THAT THE MAGNITUDE OF OUR TOTAL MOVE WAS SIGNIFICANT
C
      RATIO = (FCT-FCTLOW) / AMAX1( ABS(FCT),ABS(FCTLOW) )
      IF (-RATIO.LE.DELTA) GO TO 200
      STATUS(1) = 5H INCR
      STATUS(2) = 5HEASE 
      GO TO 2000
C
  200 TMPGDG = 0.
      TOTMVE = 0.
      DO 202 I=1,3
      DO 202 J=1,N
      TOTMVE = TOTMVE + (P(I,J)-X(I,J))**2
      TMPGDG = TMPGDG + G(I,J)**2
  202 CONTINUE
      TOTMVE = SQRT(TOTMVE)
      IF (TOTMVE.GE.EPSLN) GO TO 220
      STATUS(1) = 5H SMAL
      STATUS(2) = 5HLMOVE
      GO TO 2000
C
C   SUCCESSFUL ITERATION COMPLETE: UPDATE VARIABLES
C
  220 NITN = NITN+1
      OLDFCT = FCT
      FCT = FCTLOW
      GDG = TMPGDG
      GNM = SQRT(GDG)
C
C   OUTPUT INTERMEDIATE RESULTS EVERY IFREQ ITERATIONS
C
      IF (IFREQ.NE.0 .AND. ((ITN-1)/IFREQ)*IFREQ.EQ.ITN-1.AND.DUMP)
     1 CALL PRINT (N,X,ITN,FCT,GNM,ANGLE,TOTMVE,NFCT,NITN,IFREQ)
C
C   TEST FOR NORMAL TERMINATION OR TOO MANY ITERATIONS HERE
C
      IF (FCT.LE.FCTMIN .OR. GNM.LE.GRDMIN) GO TO 3000
      IF (NITN.LE.ITNLIM) GO TO 300
      STATUS(1) = 5H OVER
      STATUS(2) = 5HLIMIT
      GO TO 2020
C
C   TEST TO SEE IF WE SHOULD RESTART (SOME MINOR GLICH, TOO MANY CONJU-
C   GATE STEPS,PROGRESS FAST IN STEEP DIRECTION OR SLOW IN CG DIRECTION)
C
  300 STATUS(1) = 5H FAST
      STATUS(2) = 5HDECR 
      IF (ITN.EQ.1       .AND. RATIO.GE.FAST) GO TO 303
      STATUS(1) = 5H SLOW
      STATUS(2) = 5HDECR 
      IF (ITN.GT.(3*N/10).AND. RATIO.LE.SLOW) GO TO 303
      GO TO 1000
C
C   NORMAL RESTART: PROCEED FROM CURRENT POINT IN STEEPEST DIRECTION
C
  303 IF (IFREQ.NE.0.AND.DUMP) WRITE(20,330) ITN,(STATUS(NNN),NNN=1,2)
  330 FORMAT(/,' *** NORMAL RESTART AFTER ',I3,' CONJUGATE STEPS BECAUSE
     1 OF',2A5,' ***',/)
      GO TO 10
C
C   *******************************************************************
C   END OF MAIN LOOP END OF MAIN LOOP END OF MAIN LOOP END OF MAIN LOOP
C   *******************************************************************
C
 1000 CONTINUE
      STATUS(1) = 5H PERI
      STATUS(2) = 5HODIC 
      IF (IFREQ.NE.0.AND.DUMP) WRITE(20,330) ITN,(STATUS(NNN),NNN=1,2)
      GO TO 10
C
C   ******************************************************************
C   ERROR CONTROL SECTION: RESTART OR RETURN IF WE HAVE JUST RESTARTED
C   ******************************************************************
C
 2000 DO 2002 I=1,3
      DO 2002 J=1,N
      X(I,J) = P(I,J)
      G(I,J) = Q(I,J)
 2002 CONTINUE
      IF (ITN.NE.1) GO TO 2200
 2020 IF (IFREQ.NE.0.AND.DUMP) WRITE(20,2022) (STATUS(NNN), NNN=1,2)
 2022 FORMAT(/,' ***  ERROR  RETURN BECAUSE OF  ',2A5,' ***',/)
      FCTVAL = FCT
      GRDNRM = GNM
      RETURN
 2200 IF (IFREQ.NE.0.AND.DUMP) WRITE(20,2202) ITN,(STATUS(NNN),NNN=1,2)
 2202 FORMAT(/,' ***  ERROR RESTART AFTER ',I3,' CONJUGATE STEPS BECAUSE
     1 OF',2A5,' ***',/)
      GO TO 10
C
C   ****************************************************************
C   NORMAL TERMINATION FROM HERE NORMAL TERMINATION FROM HERE NORMAL
C   ****************************************************************
C
 3000 IF (IFREQ.NE.0.AND.DUMP) WRITE(20,3003)
 3003 FORMAT(/,' *** NORMAL TERMINATION SCCG ***',/)
      FCTVAL = FCT
      GRDNRM = GNM
      STATUS(1) = BLANK
      STATUS(2) = BLANK
      RETURN
      END
C
C   ****************************************************************
      REAL FUNCTION COEFCT (FCALC,NP,X,FCT,GDG,NFCT,STATUS,SLFCOR)
C   ****************************************************************
C
C   RETURNS THE COEFFICIENT TO BE USED IN COMPUTING THE CONJUGATE
C   DIRECTION, AS WELL AS THE DISPLACEMENT VECTOR  P  IN /SCCG/.
C
      EXTERNAL FCALC
      LOGICAL SLFCOR
      INTEGER STATUS(2)
      COMMON /SCCGC/ P(3,72),Q(3,72),C(3,72),G(3,72)
      DIMENSION X(3,NP)
      N = NP
C
C   BEGIN BY COMPUTING THE UNCORRECTED HESTENES-STIEFEL COEFFICIENT
C   AND THE GRAM-SCHMIDT ORTHOGONALIZATION COEFFICIENT FOR P & G.
C
      GDQ = 0.
      PDQ = 0.
      PDG = 0.
      DO 1 I=1,3
      DO 1 J=1,N
      P(I,J) = X(I,J) - P(I,J)
      Q(I,J) = G(I,J) - Q(I,J)
      PDQ = PDQ + P(I,J) * Q(I,J)
      GDQ = GDQ + G(I,J) * Q(I,J)
      IF (SLFCOR) PDG = PDG + P(I,J) * G(I,J)
    1 CONTINUE
      HSCOEF = GDQ/PDQ
      IF (.NOT.SLFCOR) GO TO 110
      GSCOEF = PDG/GDG
C
C   COMPUTE THE CORRECTED DISPLACEMENT VECTOR TO BE ORTHOGONAL TO THE
C   CURRENT GRADIENT AS WOULD BE EXPECTED FROM A PERFECT LINE SEARCH
C
      DO 10 I=1,3
      DO 10 J=1,N
      P(I,J) = P(I,J) - GSCOEF * G(I,J)
      Q(I,J) = X(I,J) - P(I,J)
   10 CONTINUE
C
C   FIND THE CORRECTED GRADIENT  C  AND CORRESPONDING FUNCTION VALUE AT
C   THE CORRECTED PRIOR POINT, AND CHECK THAT THIS GIVES A DECREASE
C
      NFCT = NFCT+1
      IF (FCALC(N,Q,C,.FALSE.).LE.FCT) GO TO 100
C
C   COMPUTE THE CORRECTED HESTENES-STIEFEL COEFFICIENT AND DOT PRODS
C
      QDG = 0.
      PDC = 0.
      CDC = 0.
      PDP = 0.
      DO 11 I=1,3
      DO 11 J=1,N
      QCOR = G(I,J) - C(I,J)
      QDG = QDG + QCOR * G(I,J)
      PDC = PDC + P(I,J)*C(I,J)
      CDC = CDC + C(I,J)**2
      PDP = PDP + P(I,J)**2
   11 CONTINUE
      SCCOEF = - QDG/PDC
      PNM = SQRT(PDP)
      CNM = SQRT(CDC)
C
C   CHECK THE ANGLE BETWEEN THE CORRECTED DISPLACEMENT AND GRADIENT
C
      COSANG = -PDC/(PNM*CNM)
      IF (COSANG.GT.1.0) COSANG = 1.0
      ANGLE = 57.2958 * ACOS(COSANG)
      IF (ANGLE.GT.89.9) GO TO 100
C
C   RETURN WITH CORRECTED CEFFICIENT AND DISPLACEMENT VECTOR
C
      COEFCT = SCCOEF
      RETURN
C
C   IF THE CORRECTED PRIOR POINT IS UNACCEPTABLE, RESTORE THE OLD
C   DISPLACEMENT VECTOR AND RETURN THE HESTENES-STIEFEL COEFFICIENT
C
  100 DO 101 I=1,3
      DO 101 J=1,N
      P(I,J) = P(I,J) + GSCOEF * G(I,J)
  101 CONTINUE
      STATUS(1) = 5H NOCO
      STATUS(2) = 5HRRCTN
  110 COEFCT = HSCOEF
      RETURN
      END
C
C   ****************************************************************
      REAL FUNCTION SEARCH (FCALC,NFCT,N0,X,FCT,OLDFCT,SLOPE,STATUS)
C   ****************************************************************
C   THIS IS THE PARABOLIC EXTRAPOLATION / CUBIC INTERPOLATION PROC-
C   EDURE DUE TO DIXON, POWELL, WEINER AND HAVEL (1980).
C
      EXTERNAL FCALC
      INTEGER STATUS(2),RESTAR
      COMMON /SCCGC/ P(3,72),Q(3,72),C(3,72),G(3,72)
      COMMON /PARA/ GESFCT,STPMIN,STPMAX,CAPPA,DELTA,EPSLN,FAST,SLOW,
     1              ANGMAX,INTMAX,RESTAR
      REAL X(3,N0)
      N = N0
C
C   PARABOLIC EXTRAPOLATION FROM PREVIOUS FUNCTION DECREASE INITIALIZES
C
      GUESS = FCT - GESFCT * (OLDFCT - FCT)
      STEP = 2. * (GUESS - FCT) / SLOPE
      IF (STEP.GT.STPMAX) STEP = STPMAX
      IF (STEP.LT.STPMIN) STEP = STPMIN
      FCT0 = FCT
      CDG0 = SLOPE
C
C   BEGIN EXTRAPOLATION
C
    1 FCTB = FCT0
      CDGB = CDG0
      INTPLN = 0
C
C   REPLACE LAST POINT BY LATEST AND TAKE ANOTHER STEP
C
    2 FCTA = FCTB
      CDGA = CDGB
      DO 3 I=1,3
      DO 3 J=1,N
      X(I,J) = X(I,J) + STEP * C(I,J)
    3 CONTINUE
      NFCT = NFCT+1
      FCTB = FCALC (N,X,G,.FALSE.)
      CDGB = 0.
      DO 4 I=1,3
      DO 4 J=1,N
      CDGB = CDGB + C(I,J) * G(I,J)
    4 CONTINUE
C
C   IF  THE FINITE DIFFERENCE CURVATURE IS NEGATIVE  DOUBLE THE STEP;
C   ELSE IF  STEP < PARABOLIC ESTIMATE < 4*STEP  USE THIS ESTIMATE;
C   ELSE TRUNKATE TO STEP OR 4*STEP, RESP.
C
      IF (ABS(CDGB/SLOPE).LE.CAPPA .AND. FCTB.LT.FCTA) GO TO 1003
      IF (CDGA*CDGB.LT.0. .OR. FCTA.LT.FCTB) GO TO 11
      STEP = 2.0 * STEP
      IF (CDGB-CDGA.LE.0.) GO TO 5
      PARAB = (FCTA-FCTB) / (CDGB-CDGA)
      IF (PARAB.GT.2.0*STEP) PARAB = 2.0*STEP
      IF (PARAB.LT.STEP/2.0) PARAB = STEP/2.0
      STEP = PARAB
    5 IF (STEP.GT.STPMAX) STEP = STPMAX
      GO TO 2
C
C   IF  SLOPES CHANGE OR FUNCTION INCREASES  BEGIN CUBIC INTERPOLATION
C
   11 CUBE = 0.
      FCTC = 1.E35
      RRR = 3.0 * (FCTB-FCTA) / STEP
      SSS = RRR - CDGA - CDGB
      TSQ = SSS*SSS - CDGA*CDGB
      IF (TSQ.LT.0.) GO TO 1002
      TTT = SQRT(TSQ)
      CUBE = STEP * (CDGB+TTT+SSS) / (CDGB-CDGA+2.0*TTT)
      IF (CUBE.LT.0. .OR. CUBE.GT.STEP) GO TO 1002
      DO 12 I=1,3
      DO 12 J=1,N
      X(I,J) = X(I,J) - CUBE * C(I,J)
   12 CONTINUE
      NFCT = NFCT+1
      FCTC = FCALC (N,X,G,.FALSE.)
      CDGC = 0.
      DO 13 I=1,3
      DO 13 J=1,N
      CDGC = CDGC + C(I,J) * G(I,J)
   13 CONTINUE
      MAYDAY = 0
      IF (FCTC.GT.FCTA) MAYDAY = MAYDAY + 1
      IF (FCTC.GT.FCTB) MAYDAY = MAYDAY + 1
      INTPLN = INTPLN + 1
C
C   IF  THE BRACKETTING POINTS HAVE OPPOSITE SLOPE  REPLACE THE INTERP-
C   OLATED POINT BY THE BRACKET WHOSE SLOPE HAS THE SAME SIGN & REPEAT;
C   ELSE  IF THE INTERPOLATED AND BRACKETS HAVE OPPOSITE SLOPE  REPLACE
C   THE FAR BRACKET (B); ELSE IF  THE INTERPOLATED HAS LOWEST FUNCTION
C   VALUE  REPLACE THE NEAR BRACKET (A); ELSE  REPLACE B ; REPEAT
C
      IF (ABS(CDGC/SLOPE).LE.CAPPA) GO TO 1004
      CUBSTP = AMIN1( ABS(CUBE),ABS(STEP-CUBE) )
      IF (INTPLN.GE.INTMAX .OR. CUBSTP.LT.STPMIN .OR.
     1              MAYDAY.EQ.2) GO TO 101
      IF (CDGA*CDGB.GE.0.) GO TO 14
      IF (CDGA*CDGC.LT.0.) GO TO 15
      GO TO 16
   14 IF (CDGA*CDGC.LT.0.) GO TO 15
      IF (FCTC.LT.FCTA) GO TO 16
   15 FCTB = FCTC
      CDGB = CDGC
      STEP = STEP-CUBE
      GO TO 11
   16 FCTA = FCTC
      CDGA = CDGC
      STEP = CUBE
      DO 17 I=1,3
      DO 17 J=1,N
      X(I,J) = X(I,J) + CUBE * C(I,J)
   17 CONTINUE
      GO TO 11
C
C   IF  INTERPOLATION LIMIT EXCEEDED OR FUNCTION INCREASES OR THE
C   INTERPOLATION CHANGE IS INSIGNIFICANT  RESTART THE LINE SEARCH
C   FROM THE LOWEST AVAILABLE POINT WITH A MUCH SMALLER STEP
C
  101 FCT1 = AMIN1( FCTA,FCTB,FCTC )
      IF (FCT1.GT.FCT0) GO TO 1002
      IF (FCT1.EQ.FCTA) GO TO 103
      IF (FCT1.EQ.FCTB) GO TO 105
      CDG1 = CDGC
      SGN = - SIGN(1.0,CDG1)
      DO 102 I=1,3
      DO 102 J=1,N
      C(I,J) = SGN * C(I,J)
  102 CONTINUE
      GO TO 107
  103 CDG1 = CDGA
      SGN = - SIGN(1.0,CDG1)
      DO 104 I=1,3
      DO 104 J=1,3
      X(I,J) = X(I,J) + (CUBE-STEP) * C(I,J)
      C(I,J) = SGN * C(I,J)
  104 CONTINUE
      GO TO 107
  105 CDG1 = CDGB
      SGN = - SIGN(1.0,CDG1)
      DO 106 I=1,3
      DO 106 J=1,N
      X(I,J) = X(I,J) + CUBE * C(I,J)
      C(I,J) = SGN * C(I,J)
  106 CONTINUE
  107 STEP = AMAX1( CUBE,STEP-CUBE ) / 10.0
      IF (STEP.LT.STPMIN) STEP = STPMIN
      IF ((STATUS(1).EQ.5H RESE).AND.(STATUS(2).EQ.ARCH )) GO TO 1001
      STATUS(1) = 5H RESE
      STATUS(2) = 5HARCH
      FCT0 = FCT1
      CDG0 = SGN*CDG1
      GO TO 1
C
C   RETURN WITH BEST VALUE IF REINTERPOLATION FAILS
C
 1001 STATUS(1) = 5H BADI
      STATUS(2) = 5HNTPLN
      SEARCH = FCT1
      RETURN
C
C   INTPLNERR  IMPLIES A SERIOUS ERROR IN THIS ROUTINE OR YOUR GRADIENT
C
 1002 SEARCH = FCT
      CALL SPIT(FCT0,FCTA,FCTB,FCTC,CDG0,CDGA,CDGB,CDGC,STEP,CUBE,RRR,
     1          SSS,TSQ,TTT,X(1,1),G(1,1),CUBSTP,INTPLN,MAYDAY,STATUS)
      STATUS(1) = 5H INTP
      STATUS(2) = 5HLNERR
      RETURN
C
C   IF  ABS( CURRENT SLOPE / INITIAL SLOPE) < CAPPA  SUCCESSFUL RETURN
C
 1003 SEARCH = FCTB
      RETURN
 1004 SEARCH = FCTC
      RETURN
      END
C
      SUBROUTINE SPIT(FCT0,FCTA,FCTB,FCTC,CDG0,CDGA,CDGB,CDGC,STEP,CUBE,
     1                RRR,SSS,TSQ,TTT,X,G,CUBSTP,INTPLN,MAYDAY,STATUS)
      INTEGER STATUS(2)
      WRITE(20,9999) (STATUS(NNN), NNN=1,2),FCT0,FCTA,FCTB,
     1              FCTC,CDG0,CDGA,CDGB,CDGC,
     2              STEP,CUBE,RRR,SSS,TSQ,TTT,X,G,CUBSTP,INTPLN,MAYDAY
 9999 FORMAT(/,' *** ERROR IN ROUTINE SEARCH: STATUS IS ',2A5,' *** ',
     1      //,' FCT0,A,B,C,CDG0,A,B,C:     ',8E13.6,/,
     2         ' STEP,CUBE,RRR,SSS,TSQ,TTT: ',6E13.6,/,
     3         ' X,G,CUBSTP,INTPLN,MAYDAY:  ',3E13.6,2I13)
      RETURN
      END
C
C   ****************************************************************
      SUBROUTINE PRINT(N,X,ITN,FCT,GNM,ANG,TMV,NFCT,NITN,IFREQ)
C   ****************************************************************
C   OUTPUT ROUTINE FOR SCCG; DUMPS SPECIFICS ON OPTIMIZATION,
C   ALONG WITH COORDINATES IF IFREQ IS LESS THAN ZERO
C
      DIMENSION X(3,N)
      WRITE(20,1) NITN,NFCT,FCT,GNM,ANG,TMV,ITN
    1 FORMAT(' ITRN ',I3,' OF SCCG: ',I4,' FCT CALLS; ',E9.3,
     1' FCT VAL; ',E9.3,' GRAD NORM; ',F6.2,' ANGLE; ',E9.3,
     2' TOT MOVE; ',I3,' CONJ STEPS.')
      IF (IFREQ.GT.0) RETURN
      WRITE(20,2)
    2 FORMAT(/,' COORDINATES:',/)
      WRITE(20,3) (J,(X(I,J),I=1,3),J=1,N)
    3 FORMAT(3(I5,1X,3F12.3))
      WRITE(20,4)
    4 FORMAT(/)
      RETURN
      END
