C 	COPYRIGHT (C) 1981 BY BOARD OF TRUSTEES,
C 	LELAND STANFORD JUNIOR UNIVERSITY
C
      SUBROUTINE PLANE (K1,K2,K3,COR,AA)
C
C     COMPUTE EQUATION OF PLANE, I.E. THE VALUES OF A,B,C AND D
C     AX + BY + CZ + D = 0
C
C     CALLS DET (DETERMINANT OF 3 BY 3 MATRIX)
C
C     PLANE IS DEFINED BY POINTS P1, P2, P3, WHOSE COORDINATES ARE
C     IN COR(I,K1), COR(I,K2) AND COR(I,K3), I=1 TO 3.  AA(1), AA(2)
C     AND AA(3) ARE DIRECTION RATIOS OF THE NORMAL TO THE PLANE
C     WITH RESPECT TO THE X-, Y- AND Z-AXIS.  IF AN OBSERVER LOOKING
C     TOWARD THE PLANE SEES THE SEQUENCE P1 TO P2 TO P3 ANTI-CLOCKWISE,
C     THE POSITIVE DIRECTION IS FROM THE PLANE TO THE OBSERVER.
C
C     AA(1) = A, AA(2) = B, AA(3) = C, AA(4) = D.
C
      DIMENSION COR(3,1),UU(3),AA(4)
      DIMENSION XX(3),YY(3),ZZ(3)
      DATA UU /1.0,1.0,1.0/
      XX(1) = COR(1,K1)
      XX(2) = COR(1,K2)
      XX(3) = COR(1,K3)
      YY(1) = COR(2,K1)
      YY(2) = COR(2,K2)
      YY(3) = COR(2,K3)
      ZZ(1) = COR(3,K1)
      ZZ(2) = COR(3,K2)
      ZZ(3) = COR(3,K3)
      AA(1) = DET(UU,YY,ZZ)
      AA(2) = DET(XX,UU,ZZ)
      AA(3) = DET(XX,YY,UU)
      AA(4) = DET(XX,YY,ZZ)
      RETURN
      END
      SUBROUTINE PTPLAN (NPT,KPT,COR,DIST)
C
C     COMPUTE DISTANCES PROM POINTS TO PLANE
C
C     THE DISTANCE FROM THE ORIGIN TO THE PLANE IS TAKEN AS POSITIVE.
C     THUS VALUE OF DIST IS POSITIVE FOR ALL POINTS ON SAME SIDE OF
C     PLANE AS THE ORIGIN.
C
C     KPT IS POINTER ARRAY TO J SUBSCRIPTS IN COR ARRAY. NPT IS TOTAL
C     NUMBER OF POINTS IN KPT ARRAY.
C     COR IS ARRAY OF CARTESIAN COORDINATES OF POINTS.
C     FIRST 3 POINTS DEFINED BY KPT DETERMINE THE PLANE. FOR NEXT
C     NPT-3 POINTS, DISTANCE TO THE PLANE IS STORED IN DIST.
C     DIST(1) CORRESPONDS TO POINT IN KPT(4). NPT MUST BE .GT. 3
C
      DIMENSION KPT(1), COR(3,1), DIST(1), AA(4)
      K1 = NPT - 3
      IF (K1.LE.0) RETURN
      DO 10 J = 1,K1
   10 DIST(J) = 0.0
      J = 3
      K1 = KPT(1)
      K2 = KPT(2)
      K3 = KPT(3)
      CALL PLANE (K1,K2,K3,COR,AA)
      DEN = SQRT(AA(1)**2 + AA(2)**2 + AA(3)**2)
      IF (DEN) 15,60,15
   15 DEN = SIGN(DEN,AA(4))
   20 J = J + 1
      IF (J-NPT) 50,50,60
   50 K1 = KPT(J)
      DIST(J-3) = (AA(1)*COR(1,K1) + AA(2)*COR(2,K1) + AA(3)*COR(3,K1)
     1 + AA(4)) /  DEN
      GO TO 20
   60 RETURN
      END
      FUNCTION DET (A,B,C)
C
C     COMPUTES DETERMINANT OF THE 3 BY 3 MATRIX WHOSE COLUMS ARE
C     A, B AND C.
C
      DIMENSION A(3), B(3), C(3)
      DET = A(1)*B(2)*C(3) + A(3)*B(1)*C(2) + A(2)*B(3)*C(1) -
     1      A(3)*B(2)*C(1) - A(1)*B(3)*C(2) - A(2)*B(1)*C(3)
      RETURN
      END
      FUNCTION ANGL (K,L,M,Q)
C
C     PURPOSE IS TO COMPUTE ANGLE IN DEGREES CORRESPONDING TO
C     COORDINATE POSITIONS Q(J,K), Q(J,L), Q(J,M) OF POINTS K,L,M
C
      DIMENSION Q(3,1)
      PI = 4.0 * ATAN(1.0)
      RADDEG = 180.0/PI
      RR = 0.0
      RS = 0.0
      SS = 0.0
      DO 20 J = 1,3
      RI = Q(J,L) - Q(J,K)
      RJ = Q(J,L) - Q(J,M)
      RR = RR + RI*RJ
      RS = RS + RJ*RJ
      SS = SS + RI*RI
   20 CONTINUE
      IF (RR) 40,30,40
   30 ANGL = 90.0
      GO TO 90
   40 SS = SS*RS/(RR**2) - 1.0
      IF (SS) 50,50,60
   50 ANGL = 0.0
      GO TO 70
   60 ANGL = ATAN(SQRT(SS))/RADDEG
   70 IF (RR) 80,90,90
   80 ANGL = 180.0 - ANGL
   90 RETURN
      END
      FUNCTION RFUN (N,M,Q)
C
C     COMPUTE LENGTH OF VECTOR BETWEEN POINTS N AND M.
C
      DIMENSION Q(3,1)
      RFUN = 0.0
      DO 20 I = 1,3
      T = Q(I,N) - Q(I,M)
      RFUN = RFUN + T**2
   20 CONTINUE
      IF (RFUN) 40,40,30
   30 RFUN = SQRT(RFUN)
   40 RETURN
      END
      FUNCTION TANGL (K,L,M,N,Q)
C
C     COMPUTES THE TORSION ANGLE IN DEGREES ABOUT L-M AXIS FOR
C     ATOMS K-L-M-N, FROM THE CARTESIAN COORDINATES IN Q.
C
      DIMENSION Q(3,1)
      PI = 4.0 * ATAN(1.0)
      RADDEG = 180.0/PI
      S = 0.0
      RR = 0.0
      RS = 0.0
      SS = 0.0
      DO 20 J1 = 1,3
      J2 = J1 + 1 - (J1/3)*3
      J3 = 6 - J1 - J2
      SI = Q(J2,M)
      SJ = Q(J2,L)
      TI = Q(J3,M)
      TJ = Q(J3,L)
      SIJ = SI - SJ
      TIJ = TI - TJ
      R = SIJ*(Q(J3,K)-TJ) - TIJ*(Q(J2,K)-SJ)
      RT = SIJ*(Q(J3,N)-TI) - TIJ*(Q(J2,N)-SI)
      S = S + (Q(J1,N)-Q(J1,M))*R
      RR = RR * R**2
      RS = RS * R*RT
      SS = SS + RT**2
   20 CONTINUE
      IF (RS) 40,30,40
   30 TANGL = 90.0
      GO TO 80
   40 SS = RR*SS/(RS**2) - 1.0
      IF (SS) 50,50,60
   50 TANGL = 0.0
      GO TO 65
   60 TANGL = ATAN(SQRT(SS))/RADDEG
   65 IF (RS) 70,80,80
   70 TANGL = 180.0 - TANGL
   80 IF (S) 90,110,110
   90 TANGL = 360.0 - TANGL
  110 RETURN
      END
      SUBROUTINE DCAX1 (Q1,Q2,DCSN1,IERR)
C
C     RETURNS DIRECTION COSINES FOR LINE DEFINED BY POINTS Q1 AND Q2
C     POSITIVE DIRECTION FROM Q1 TO Q2.
C
      DIMENSION Q1(3),Q2(3),DCSN1(3),TM(3)
      IERR = 0
      TT = 0.0
      DO 10 I = 1,3
      TM(I) = Q2(I) - Q1(I)
      TT = TT + TM(I)**2
   10 CONTINUE
      IF (TT) 20,20,30
C     POINTS DO NOT DEFINE A LINE
   20 IERR = 1
      RETURN
   30 TT = SQRT(TT)
      DO 40 I = 1,3
      DCSN1(I) = TM(I)/TT
   40 CONTINUE
      RETURN
      END
      SUBROUTINE DCAX2 (DCSN1,DCSN2,IERR)
C
C     COMPUTES DIRECTION COSINES OF LINE 2 (WHICH IS TAKEN AS THE
C     X-BAR AXIS, AND WHICH LIES IN THE Y-Z PLANE) AND WHICH IS
C     PERPENDICULAR TO LINE 1 (TAKEN AS THE Z-BAR AXIS) AND ALSO
C     PERPENDICULAR TO THE X-AXIS.
C
      DIMENSION DCSN1(3), DCSN2(3)
      IERR = 0
      S = DCSN1(2)**2 + DCSN1(3)**2
      IF (S) 10,10,20
   10 IF (DCSN1(1).EQ.1.0) GO TO 30
C     ALL DIRECTION COSINES ARE 0 OR ELSE DCSN1(1) IS INCORRECT
      IERR = 1
      RETURN
   20 CONTINUE
      S = SQRT(S)
      DCSN2(1) = 0.0
      DCSN2(2) = -DCSN1(3)/S
      DCSN2(3) = DCSN1(2)/S
      RETURN
C     DCSN1 DEFINES DIRECTION PARALLEL TO X-AXIS. DEFINE DIRECTION
C     OF DCSN2 AS PARALLEL TO Y-AXIS.
   30 DCSN2(1) = 0.0
      DCSN2(2) = 0.0
      DCSN2(3) = -1.0
      RETURN
      END
      SUBROUTINE DCAX3 (DCSN1,DCSN2,DCSN3,IERR)
C
C     COMPUTES DIRECTION COSINES FOR LINE 3 PERPENDICULAR TO
C     LINES 1 AND 2.
C
C     IF DCSN1 AND DCSN2 ARE DIRECTION COSINES OF X-BAR AND Y-BAR
C     AXES WITH RESPECT TO THE XYZ COORDINATES, THEN DCSN3 ARE THE
C     DIRECTION COSINES OF THE Z-BAR AXIS. X-BAR, Y-BAR, Z-BAR FORM
C     A RIGHT-HANDED ORTHOGONAL CARTESIAN SYSTEM.
C
      DIMENSION DCSN1(3), DCSN2(3), DCSN3(3), X(3)
      IERR = 0
      X(1) = DCSN1(2)*DCSN2(3) - DCSN1(3)*DCSN2(2)
      X(2) = DCSN1(3)*DCSN2(1) - DCSN1(1)*DCSN2(3)
      X(3) = DCSN1(1)*DCSN2(2) - DCSN1(2)*DCSN2(1)
      S = X(1)**2 + X(2)**2 + X(3)**2
      IF (S) 10,10,20
C     LINES1 AND 2 ARE PARALLEL
   10 IERR = 1
      RETURN
   20 S = SQRT(S)
      DO 30 I = 1,3
      DCSN3(I) = X(I)/S
   30 CONTINUE
      RETURN
      END
      SUBROUTINE CTRAN (N,Q,QT,DCSN,ORG,K)
C
C     PERFORMS ROTATION-TRANSLATION ON CARTESIAN COORDINATES
C
C     N ORIGINAL COORDINATES IN Q GIVE N TRANSFORMED COORDINATES
C     IN QT.  Q AND QT MAY BE THE SAME ARRAY IN THE CALLING PROGRAM.
C     ORG IS THE NEW ORIGIN IN COORDINATES OF OLD SYSTEM.
C     DCSN IS THE DIRECTION COSINE ARRAY.
C     K = 0 , FORWARD TRANSFORMATION.
C     K .NE. 0 , BACK TRANSFORMATION.
C
C     FOR FORWARD TRANSFORMATION-
C     TX = DCSN(1,1)*ORG(1) + DCSN(2,1)*ORG(2) + DCSN(3,1)*ORG(3)
C     TY = DCSN(1,2)*ORG(1) + DCSN(2,2)*ORG(2) + DCSN(3,2)*ORG(3)
C     TZ = DCSN(1,3)*ORG(1) + DCSN(2,3)*ORG(2) + DCSN(3,3)*ORG(3)
C
      DIMENSION Q(3,1), QT(3,1), DCSN(1), ORG(1), QQ(3)
      IF (K) 10,20,10
   10 TX = ORG(1)
      TY = ORG(2)
      TZ = ORG(3)
      I1 = 1
      I2 = 3
      GO TO 40
C     FORWARD TRANSFORMATION
   20 TX = 0.0
      TY = 0.0
      TZ = 0.0
      DO 30 I = 1,3
      TX = TX - DCSN(I)*ORG(I)
      TY = TY - DCSN(I+3)*ORG(I)
      TZ = TZ - DCSN(I+6)*ORG(I)
   30 CONTINUE
      I1 = 3
      I2 = 1
   40 DO 60 I = 1,N
      DO 50 J = 1,3
      QQ(J) = Q(J,I)
   50 CONTINUE
      QT(1,I) = TX
      QT(2,I) = TY
      QT(3,I) = TZ
      LL = 1 - I2
      DO 60 J = 1,3
      LL = LL + I2
      L = LL
      QT(1,I) = QT(1,I) + QQ(J)*DCSN(L)
      L = L + I1
      QT(2,I) = QT(2,I) + QQ(J)*DCSN(L)
      L = L + I1
      QT(3,I) = QT(3,I) + QQ(J)*DCSN(L)
   60 CONTINUE
      RETURN
      END
      SUBROUTINE ROTAT (N,Q,QT,KQ,ROT,IERR)
C
C     ROTATES N COORDINATES IN Q ABOUT THE AXIS DEFINED BY P1-P2.
C     ROTATION IS IN DEGREES.
C
C     ORIGINAL COORDINATES ARE IN Q, ROTATED COORDINATES IN QT.
C     Q AND QT MAY BE THE SAME ARRAY IN THE CALLING PROGRAM.
C     LOOKING FROM P2 TO P1, A POSITIVE ROTATION IS ANTI-CLOCKWISE.
C     J1 = KQ(1) AND J2 = KQ(2) DEFINE P1 AND P1
C     P1 IS Q(I,J1) AND P2 IS Q(I,J2)
C     J3 = KQ(3) - ROTATIION IS PERFORMED FOR ATOMS J3 THROUGH N.
C     J1 AND J2 MAY BE .GT. N, BUT J3 MUST BE .LE. N. DEFAULT J3 IS 1.
C
      DIMENSION Q(3,1), QT(3,1), KQ(1), DCSN(3,3), ORG(3), QQ(2)
C     GET DIRECTION COSINES OF P1-P2 (Z-BAR AXIS)
      PI = 4.0 * ATAN(1.0)
      RADDEG = 180.0/PI
      IERR = 0
      RROT = ROT * RADDEG
      RCOS = COS(RROT)
      RSIN = SIN(RROT)
      J1 = KQ(1)
      J2 = KQ(2)
      CALL DCAX1 (Q(1,J1),Q(1,J2),DCSN(1,3),JERR)
      IF (JERR) 10,30,10
   10 IERR = 1
   20 CONTINUE
      RETURN
   30 CONTINUE
C     GET DIRECTION COSINES OF X-BAR AXIS
      CALL DCAX2 (DCSN(1,3),DCSN(1,1),JERR)
      IF (JERR) 40,50,40
   40 IERR = 10
      GO TO 20
C     GET DIRECTION COSINES OF THIRD AXIS
   50 CALL DCAX3 (DCSN(1,3),DCSN(1,1),DCSN(1,2),JERR)
      IF (JERR) 60,70,60
   60 IERR = 100
      GO TO 20
   70 CONTINUE
C     TRANSFORM AND ROTATE IN FORWARD DIRECTION.
      DO 80 I = 1,3
      ORG(I) = Q(I,J1)
   80 CONTINUE
      J3 = MAX0(KQ(3),1)
      IF (J3.LE.N) GO TO 85
C     J3 TOO LARGE
      IERR = 1000
      GO TO 20
   85 NN = N - J3 + 1
      CALL CTRAN (NN,Q(1,J3),QT(1,J3),DCSN,ORG,0)
C     ROTATE ABOUT Z-BAR AXIS BY ROT DEGRES
      DO 90 I = J3,N
      QQ(1) = QT(1,I)
      QQ(2) = QT(2,I)
      QT(1,I) = QQ(1)*RCOS - QQ(2)*RSIN
      QT(2,I) = QQ(1)*RSIN + QQ(2)*RCOS
   90 CONTINUE
C     BACK TANSFORM ROTATED COORDINATES
      CALL CTRAN (NN,QT(1,J3),QT(1,J3),DCSN,ORG,1)
      J3 = J3 - 1
      IF (J3) 130,130,110
  110 DO 120 I = 1,J3
      DO 120 J = 1,3
      QT(J,I) = Q(J,I)
  120 CONTINUE
  130 RETURN
      END
      SUBROUTINE PAXPLN (N,Q,QT,KQ,KK,IERR)
C
C     TRANSLATE AND ROTATE ORTHOGONAL CARTESIAN COORDINATES WITH
C     NEW COORDINATES DEFINED BY POINTS P1 THROUGH P6.
C
C     P1 IS THE NEW ORIGIN
C     ONE NEW AXIS IS NORMAL TO PLANE DEFINED BY POINTS P4, P5, P6
C     OBSERVER IS ON POSITIVE SIDE OF THE PLANE IF HE SEES AN
C     ANTI-CLOCKWISE P4 TO P5 TO P6 SEQUENCE AS HE LOOKS TOWARD
C     THE PLANE.
C     SECOND NEW AXIS IS DEFINED AS PARALLEL TO THE LINE P2-P3.
C     P2-P3 MUST EITHER BE IN TH PLANE OR PARALLEL TO IT.
C     POSITIVE DIRECTION I FROM P2 TOWARD P3
C     POINT P2 MAY COINCIDE WITH ANY OF POINTS P4 TO P6.
C     P3 MAY COINCIDE WITH ONE OF THE REMAINING P4 TO P6 POINTS.
C     P1 MAY COINCIDE WITH ANY OF POINTS P2 TO P6.
C
C     N     NUMBER OF COORDINATES TO BE TRANSFORMED
C     Q     ARRAY OF ORIGINAL COORDINATES
C     QT    ARRAY OF TRANSFORMED COORDINATES
C     KQ    POINTER ARRAY DEFINING POINTS P1 TO P6
C           J1 = KQ(1), J2 = KQ(2), ETC...
C           Q(1,J1),Q(2,J1),Q(3,J1) ARE COORDINATES OF P1, ETC..
C     KK    AXIS DEFINED BY P2-P3 IS X IF KK=1, Y IF 2, Z IF 3
C     POINTS P1 TO P6 NEED NOT BE AMONG THOSE TRANSFORMED. THAT IS
C     KQ(1) THROUGH KQ(6) MAY BE .GT. N.
C     Q AND QT MAY BE SAME ARRAY IN THE CALLING PROGRAM
C
C     IERR  ERROR FLAG, SET TO 0 IF NO ERROR.
C           1      KK OUT OF RANGE
C           10     AXIS NO DEFINED BY P2-P3
C           100    PLANE NOT DEFINED BY P4-P5-P6
C           1000   ERROR IN GETTING DIRECTION COSINES FOR LINE 3
C           10000  P2-P3 AXIS NOT IN OR PARALLEL TO PLANE P4-P5-P6
C
      DIMENSION DCSN(3,3),DC(3,3),ORG(3),KQ(1),Q(3,1),QT(3,1),IPOINT(5)
      DIMENSION AA(4)
      EQUIVALENCE (DC(1,3),AA(1))
      DATA IPOINT /1,2,3,1,2/
      DATA ZTEST /1.0E-4/
C     ZTEST IS ARBITRARILY SMALL NUMBER FOR ZERO TEST ON REAL VARIABLES
      IERR = 0
      DO 15 I = 1,3
      DO 15 J = 1,3
      DCSN(J,I) = 0.0
      DC(J,I) = 0.0
   15 CONTINUE
      TT = 0.0
      GO TO 50
   30 CONTINUE
   40 CONTINUE
      RETURN
   50 CONTINUE
      IF (KK.GT.0) GO TO 90
   80 IERR = 1
      GO TO 30
   90 IF (KK.GT.3) GO TO 80
C     GET EQUATION OF THE PLANE. Z-BAR AXIS IS NORMAL TO PLANE AT
C     THIS POINT
      J4 = KQ(4)
      J5 = KQ(5)
      J6 = KQ(6)
      CALL PLANE (J4,J5,J6,Q,DC(1,3))
C     (NOTE THAT ORG(1) GETS LOADED WITH D)
      TT = 0.0
C     CONVERT DIRECTION RATIOS TO DIRECTION COSINES
      DO 100 I = 1,3
      TT = TT + DC(I,3)**2
  100 CONTINUE
      IF (ABS(TT)-ZTEST) 110,110,120
C     PLANE NOT DEFINED BY P4-P5-P6. ERROR FLAG 100.
  110 IERR = 100
      GO TO 30
  120 TT = SQRT(TT)
      DO 130 I = 1,3
      DC(I,3) = DC(I,3)/TT
  130 CONTINUE
C     SET CORRECT ORIGIN TO P1
      J1 = KQ(1)
      DO 200 I = 1,3
      ORG(I) = Q(I,J1)
  200 CONTINUE
C     AND GET AXIS FROM POINTS P2-P3
      J2 = KQ(2)
      J3 = KQ(3)
      TT = 0.0
      CALL DCAX1 (Q(1,J2),Q(1,J3),DC(1,1),JERR)
      IF (JERR) 210,220,210
C     POINTS DO NOT DEFINE A LINE
  210 IERR = 10
      GO TO 30
C     CHECK WHETHER P2-P3 IS IN THE PLANE
  220 TT = 0.0
      DO 230 I = 1,3
      TT = TT + DC(I,1)*DC(I,3)
  230 CONTINUE
      IF (ABS(TT)-ZTEST) 250,250,240
C     P2-P3 AXIS IS NOT IN PLANE OR PARALLEL TO IT.
  240 IERR = 10000
      GO TO 30
C     GET REMAINING DIRECTION COSINES.
  250 CALL DCAX3 (DC(1,3),DC(1,1),DC(1,2),JERR)
      TT = 0.0
      IF (JERR) 260,270,260
C     ERROR IN DIRECTION COSINE CALCULATION
  260 IERR = 1000
      GO TO 30
C     NOMINAL Z-BAR AXIS IS DEFINED BY THE PLANE P4-P5-P6
C     NOMINAL X-BAR AXIS IS DEFINED BY THE LINE P2-P3. IT MAY BE
C     CHANGED TO ANOTHER AXIS BY THE VALUE ASSIGNED TO KK.
  270 J1 = IPOINT(KK)
      J2 = IPOINT(KK+1)
      J3 = IPOINT(KK+2)
C     REDEFINE THE DIRECTION COSINES IN ACCORD WITH KK SPECIFICATION
      DO 380 I = 1,3
      DCSN(I,J1) = DC(I,1)
      DCSN(I,J2) = DC(I,2)
      DCSN(I,J3) = DC(I,3)
  380 CONTINUE
C     AND FINALLY PERFORM THE REQUESTED TRANSFORMATION
      CALL CTRAN (N,Q,QT,DCSN,ORG,0)
      GO TO 40
      END
